@@ -15,8 +15,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -68,6 +68,7 @@ class PreviewPanoMaskTool;
 class PreviewControlPointTool;
 class PreviewLayoutLinesTool;
 class PreviewColorPickerTool;
+class PreviewEditCPTool;
 
 class PanosphereOverviewProjectionGridTool;
 class PreviewProjectionGridTool;
@@ -87,6 +88,7 @@ class GLwxAuiFloatingFrame;
 #include <wx/frame.h>
 #include <wx/aui/aui.h>
 #include <wx/clrpicker.h>
+#include "hugin/GuiLevel.h"
 
 #include <iostream>
 
@@ -97,7 +99,7 @@ class ImageToogleButtonEventHandler : public wxEvtHandler
 public:
     ImageToogleButtonEventHandler(unsigned int image_number,
                                   wxToolBarToolBase* identify_toolbutton_in,
-                                  PT::Panorama * m_pano);
+                                  HuginBase::Panorama * m_pano);
     void OnChange(wxCommandEvent &e);
     void AddIdentifyTool(PreviewIdentifyTool** identify_tool_in);
 protected:
@@ -108,13 +110,13 @@ private:
     unsigned int image_number;
     std::vector<PreviewIdentifyTool**> identify_tools;
     wxToolBarToolBase *identify_toolbutton;
-    PT::Panorama * m_pano;
+    HuginBase::Panorama * m_pano;
 };
 
 class ImageGroupButtonEventHandler : public wxEvtHandler
 {
 public:
-    ImageGroupButtonEventHandler(unsigned int image_number, GLPreviewFrame* frame_in, PT::Panorama* m_pano);
+    ImageGroupButtonEventHandler(unsigned int image_number, GLPreviewFrame* frame_in, HuginBase::Panorama* m_pano);
     void OnChange(wxCommandEvent &e);
     void AddDragTool(DragTool** drag_tool_in);
     void AddIdentifyTool(PreviewIdentifyTool** identify_tool_in);
@@ -127,7 +129,7 @@ private:
     std::vector<DragTool**> drag_tools;
     std::vector<PreviewIdentifyTool**> identify_tools;
     GLPreviewFrame* frame;
-    PT::Panorama * m_pano;
+    HuginBase::Panorama * m_pano;
 };
 
 
@@ -182,22 +184,20 @@ private:
  *  it is not created with XRC, because it is highly dynamic, buttons
  *  have to be added etc.
  */
-class GLPreviewFrame : public wxFrame, public PT::PanoramaObserver, public AppBase::MultiProgressDisplay
+class GLPreviewFrame : public wxFrame, public HuginBase::PanoramaObserver
 {
 public:
 
     /** ctor.
      */
-    GLPreviewFrame(wxFrame * frame, PT::Panorama &pano);
+    GLPreviewFrame(wxFrame * frame, HuginBase::Panorama &pano);
 
     /** dtor.
      */
     virtual ~GLPreviewFrame();
 
-    void panoramaChanged(PT::Panorama &pano);
-    void panoramaImagesChanged(PT::Panorama &pano, const PT::UIntSet &changed);
-    
-    void updateProgressDisplay();
+    virtual void panoramaChanged(HuginBase::Panorama &pano);
+    virtual void panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet &changed);
     
     void MakePreviewTools(PreviewToolHelper * helper);
     void MakePanosphereOverviewTools(PanosphereOverviewToolHelper * helper);
@@ -213,6 +213,8 @@ public:
     void LoadOpenGLLayout();
     /** init previews */
     void InitPreviews();
+    /** sets the gui level */
+    void SetGuiLevel(GuiLevel newLevel);
 
     GLwxAuiManager* getAuiManager() {return m_mgr;}
     GLPreview* getPreview() {return m_GLPreview;}
@@ -237,8 +239,8 @@ public:
     void ToggleImageInDragGroup(unsigned int image_nr, bool update_check_box = true);
     void RemoveImageFromDragGroup(unsigned int image_nr, bool update_check_box = true);
     void AddImageToDragGroup(unsigned int image_nr, bool update_check_box = true);
-    void SetDragGroupImages(PT::UIntSet imageDragGroup_in, bool update_check_box = true);
-    PT::UIntSet GetDragGroupImages();
+    void SetDragGroupImages(HuginBase::UIntSet imageDragGroup_in, bool update_check_box = true);
+    HuginBase::UIntSet GetDragGroupImages();
     void ClearDragGroupImages(bool update_check_box = true);
     /** updates the global white balance 
      * @param redFactor multiplies all WhiteBalanceRed of individuel images with this factor
@@ -265,6 +267,7 @@ protected:
     void OnPhotometric(wxCommandEvent & e);
     void OnIdentify(wxCommandEvent &e);
     void OnAutocrop(wxCommandEvent &e);
+    void OnStackAutocrop(wxCommandEvent &e);
     void OnControlPoint(wxCommandEvent &e);
     void OnNumTransform(wxCommandEvent & e);
     void OnChangeFOV(wxScrollEvent & e);
@@ -275,6 +278,8 @@ protected:
     void OnProjParameterReset(wxCommandEvent & e);
     /** event handler for switch on/off grid on preview */
     void OnSwitchPreviewGrid(wxCommandEvent & e);
+    /** user wants to quit program */
+    void OnUserExit(wxCommandEvent & e);
 
     void OnDefaultExposure( wxCommandEvent & e );
     void OnDecreaseExposure( wxSpinEvent & e );
@@ -305,10 +310,6 @@ protected:
     void updatePano();
     /** event handler for full screen */
     void OnFullScreen(wxCommandEvent &e);
-    /** event handler for undo */
-    void OnUndo(wxCommandEvent &e);
-    /** event handler for redo */
-    void OnRedo(wxCommandEvent &e);
     /** event handler for selection of new mode */
     void OnSelectMode(wxNotebookEvent &e);
     /** event handler for blocking changing mode when panorama contains no images*/
@@ -317,11 +318,37 @@ protected:
     void OnLayoutScaleChange(wxScrollEvent &e);
     /** event handler when starting color picker */
     void OnColorPicker(wxCommandEvent &e);
+    /** event handler when starting edit cp tool */
+    void OnEditCPTool(wxCommandEvent &e);
     /** event handler when user changes background color */
     void OnPreviewBackgroundColorChanged(wxColourPickerEvent & e);
     /** event handler when user selects different guide */
     void OnGuideChanged(wxCommandEvent &e);
-    
+    /** event handler to show main frame */
+    void OnShowMainFrame(wxCommandEvent &e);
+    // assistant related event handler
+    void OnLoadImages( wxCommandEvent & e );
+    void OnAlign( wxCommandEvent & e );
+    void OnCreate( wxCommandEvent & e );
+    void OnLensTypeChanged (wxCommandEvent & e);
+    void OnFocalLengthChanged(wxCommandEvent & e);
+    void OnCropFactorChanged(wxCommandEvent & e);
+
+    /** handler for creating cp in pano space */
+    void OnCreateCP(wxCommandEvent & e);
+    /** handler to remove cp */
+    void OnRemoveCP(wxCommandEvent & e);
+    /** handle menu close event */
+    void OnMenuClose(wxMenuEvent & e);
+    /** handle context menu on select all button */
+    void OnSelectContextMenu(wxContextMenuEvent& e);
+    /** handle all options of select all context menu */
+    void OnSelectAllMenu(wxCommandEvent& e);
+    void OnSelectMedianMenu(wxCommandEvent& e);
+    void OnSelectDarkestMenu(wxCommandEvent& e);
+    void OnSelectBrightestMenu(wxCommandEvent& e);
+    void OnSelectKeepSelection(wxCommandEvent& e);
+    void OnSelectResetSelection(wxCommandEvent& e);
 private:
     /** changes the visibility of the group check boxes
      * @param isShown true if the group checkboxes should be visible
@@ -332,20 +359,26 @@ private:
      *  @return true, of mode was successful changed to desired mode
      */
     bool UpdateOverviewMode(int newMode);
+    /** update display of ROI */
+    void UpdateRoiDisplay(const HuginBase::PanoramaOptions opts);
     /** The dock manager */
     GLwxAuiManager * m_mgr;
 
     void SetMode(int newMode);
-    PT::Panorama & m_pano;
+    HuginBase::Panorama & m_pano;
 
     GLPreview * m_GLPreview;
     GLOverview * m_GLOverview;
 
+    GuiLevel m_guiLevel;
+
     ViewState* m_view_state;
 
     int m_mode;
     int non_layout_blend_mode;
     wxToolBar* m_ToolBar_Identify;
+    wxToolBar* m_ToolBar_ColorPicker;
+    wxToolBar* m_ToolBar_editCP;
     wxNotebook* m_tool_notebook;
     wxPanel* m_projection_panel;
     wxSlider * m_HFOVSlider;
@@ -367,8 +400,8 @@ private:
     // wxChoice * m_outputModeChoice;
     wxTextCtrl * m_exposureTextCtrl;
     wxBitmapButton * m_defaultExposureBut;
+    wxTextCtrl* m_exposureText;
     wxSpinButton * m_exposureSpinBut;
-    wxCheckBox * m_previewGrid;
 #if wxCHECK_VERSION(2, 9, 1)
     /// Bar for context sensitive projection information.
     wxInfoBar * m_infoBar;
@@ -376,6 +409,33 @@ private:
     // True if the status bar text has been replaced with projection information
     bool m_projectionStatusPushed;
 #endif
+    //assistant related controls
+    wxStaticText * m_imagesText;
+    wxButton * m_alignButton;
+    wxButton * m_createButton;
+    wxChoice   * m_lensTypeChoice;
+    wxTextCtrl * m_focalLengthText;
+    wxTextCtrl * m_cropFactorText;
+    wxButton   * m_loadLensButton;
+    wxMenu* m_filemenuSimple;
+    wxMenu* m_filemenuAdvanced;
+    wxMenu* m_selectAllMenu;
+#if wxCHECK_VERSION(2,9,2)
+    wxButton* m_selectAllButton;
+#else
+    wxBitmapButton* m_selectAllButton;
+#endif
+    enum SelectAllMode
+    {
+        SELECT_ALL_IMAGES = 0,
+        SELECT_MEDIAN_IMAGES = 1,
+        SELECT_BRIGHTEST_IMAGES = 2,
+        SELECT_DARKEST_IMAGES = 3,
+    };
+    SelectAllMode m_selectAllMode;
+    bool m_selectKeepSelection;
+
+    int m_degDigits;
 
     wxColour m_preview_background_color;
 
@@ -384,9 +444,9 @@ private:
     // index of difference mode
     int m_differenceIndex;
 
-	  wxScrolledWindow * m_ButtonPanel;
-	  wxBoxSizer * m_ButtonSizer;
-	  wxStaticBoxSizer * m_ToggleButtonSizer;
+    wxScrolledWindow * m_ButtonPanel;
+    wxBoxSizer * m_ButtonSizer;
+    wxStaticBoxSizer * m_ToggleButtonSizer;
 
     wxBoxSizer * m_topsizer;
     wxBoxSizer * m_projParamSizer;
@@ -404,17 +464,16 @@ private:
     std::vector<ImageToogleButtonEventHandler *> toogle_button_event_handlers;
     std::vector<ImageGroupButtonEventHandler *> toggle_group_button_event_handlers;
 
-    wxToggleButton * m_OverviewToggle;
-
     DECLARE_EVENT_TABLE()
 
     // tools
     PreviewToolHelper *preview_helper;
     
     PreviewCropTool *crop_tool;
-    PT::UIntSet imageDragGroup;
+    HuginBase::UIntSet imageDragGroup;
     PreviewDragTool *drag_tool;
     PreviewColorPickerTool *color_picker_tool;
+    PreviewEditCPTool* edit_cp_tool;
 
     PreviewIdentifyTool *identify_tool;
     PreviewIdentifyTool *panosphere_overview_identify_tool;
@@ -459,12 +518,12 @@ private:
     bool loadedLayout;
 #endif
 
-    /** check, if panorama has non-zero translation parameters
+    /** check, if panorama has non-zero translation plane parameters
      *  @returns true, if panorama has non-zero translation parameters
      */
-    bool HasNonZeroTranslationParameters();
-    /** resets all translation parameters to zero */
-    void ResetTranslationParameters();
+    bool HasNonZeroTranslationPlaneParameters();
+    /** resets all translation plane parameters to zero */
+    void ResetTranslationPlaneParameters();
     void CleanButtonColours();
     /** Tell the user anything suspicious about the projection choice.
      * If nothing is suspicious, any previous message is removed.
