@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -43,7 +43,12 @@
 #include "hugin/huginApp.h"
 #include "hugin/PreviewPanel.h"
 #include "hugin/ImagesPanel.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
+#include "hugin/TextKillFocusHandler.h"
+#include "algorithms/nona/FitPanorama.h"
+#include "algorithms/basic/CalculateMeanExposure.h"
+#include "base_wx/wxPlatform.h"
 
 #include <vigra_ext/ImageTransforms.h>
 
@@ -51,8 +56,6 @@ extern "C" {
 #include <pano13/queryfeature.h>
 }
 
-using namespace hugin_utils;
-
 // a random id, hope this doesn't break something..
 enum {
     ID_PROJECTION_CHOICE = wxID_HIGHEST +1,
@@ -96,16 +99,18 @@ BEGIN_EVENT_TABLE(PreviewFrame, wxFrame)
 #else
     EVT_CHECKBOX(-1, PreviewFrame::OnChangeDisplayedImgs)
 #endif
-
 #ifndef __WXMAC__
-    // wxMac does not process these
-    EVT_SCROLL_CHANGED(PreviewFrame::OnChangeFOV)
+	EVT_SCROLL_CHANGED(PreviewFrame::OnChangeFOV)
 #else
-    EVT_SCROLL_THUMBRELEASE(PreviewFrame::OnChangeFOV)
-    EVT_SCROLL_ENDSCROLL(PreviewFrame::OnChangeFOV)
-    EVT_SCROLL_THUMBTRACK(PreviewFrame::OnChangeFOV)
+ #if wxCHECK_VERSION(2,9,0)
+	EVT_SCROLL_CHANGED(PreviewFrame::OnChangeFOV)
+ #else
+	EVT_SCROLL_THUMBRELEASE(PreviewFrame::OnChangeFOV)
+	EVT_SCROLL_ENDSCROLL(PreviewFrame::OnChangeFOV)
+	EVT_SCROLL_THUMBTRACK(PreviewFrame::OnChangeFOV)
+ #endif
 #endif
-    EVT_TOOL(ID_FULL_SCREEN, PreviewFrame::OnFullScreen)
+	EVT_TOOL(ID_FULL_SCREEN, PreviewFrame::OnFullScreen)
     EVT_TOOL(ID_UNDO, PreviewFrame::OnUndo)
     EVT_TOOL(ID_REDO, PreviewFrame::OnRedo)
     EVT_BUTTON(PROJ_PARAM_RESET_ID, PreviewFrame::OnProjParameterReset)
@@ -113,7 +118,7 @@ END_EVENT_TABLE()
 
 #define PF_STYLE (wxMAXIMIZE_BOX | wxMINIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION | wxCLOSE_BOX | wxCLIP_CHILDREN)
 
-PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
+PreviewFrame::PreviewFrame(wxFrame * frame, HuginBase::Panorama &pano)
     : wxFrame(frame,-1, _("Panorama preview"), wxDefaultPosition, wxDefaultSize,
               PF_STYLE),
       m_pano(pano)
@@ -141,9 +146,9 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
     m_ButtonPanel->SetAutoLayout(true);
 	m_ButtonPanel->SetSizer(m_ButtonSizer);
 						
-	m_ToggleButtonSizer->Add(m_ButtonPanel, 1, wxEXPAND | wxADJUST_MINSIZE, 0);
+	m_ToggleButtonSizer->Add(m_ButtonPanel, 1, wxEXPAND, 0);
 
-    m_topsizer->Add(m_ToggleButtonSizer, 0, wxEXPAND | wxADJUST_MINSIZE | wxALL, 5);
+    m_topsizer->Add(m_ToggleButtonSizer, 0, wxEXPAND | wxALL, 5);
 
     wxFlexGridSizer * flexSizer = new wxFlexGridSizer(2,0,5,5);
     flexSizer->AddGrowableCol(0);
@@ -169,7 +174,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
                                 _("VFOV"));
     m_VFOVSlider->SetLineSize(2);
     m_VFOVSlider->SetPageSize(10);
+#if wxCHECK_VERSION(3,0,0)
+    m_VFOVSlider->SetTickFreq(5);
+#else
     m_VFOVSlider->SetTickFreq(5,0);
+#endif
     m_VFOVSlider->SetToolTip(_("drag to change the vertical field of view"));
 
     flexSizer->Add(m_VFOVSlider, 0, wxEXPAND);
@@ -182,7 +191,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
                                 _("HFOV"));
     m_HFOVSlider->SetPageSize(10);
     m_HFOVSlider->SetLineSize(2);
+#if wxCHECK_VERSION(3,0,0)
+    m_HFOVSlider->SetTickFreq(5);
+#else
     m_HFOVSlider->SetTickFreq(5,0);
+#endif
 
     m_HFOVSlider->SetToolTip(_("drag to change the horizontal field of view"));
 
@@ -276,7 +289,7 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
 //                                          wxArtProvider::GetBitmap(wxART_GO_BACK));
 //    blendModeSizer->Add(m_decExposureBut);
 
-    m_exposureTextCtrl = new wxTextCtrl(this, ID_EXPOSURE_TEXT, _("0"),
+    m_exposureTextCtrl = new wxTextCtrl(this, ID_EXPOSURE_TEXT, wxT("0"),
                                         wxDefaultPosition,wxSize(50,-1), wxTE_PROCESS_ENTER);
     blendModeSizer->Add(m_exposureTextCtrl,
                           0,        // not vertically strechable
@@ -298,7 +311,7 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
 
     wxBitmapButton * resetProjButton=new wxBitmapButton(this, PROJ_PARAM_RESET_ID, 
         wxArtProvider::GetBitmap(wxART_REDO));
-    resetProjButton->SetToolTip(_("Resets the projections parameters to their default values."));
+    resetProjButton->SetToolTip(_("Resets the projection's parameters to their default values."));
     m_projParamSizer->Add(resetProjButton, 0, wxLEFT | wxRIGHT | wxALIGN_CENTER_VERTICAL, 5);
 
     m_projParamNamesLabel.resize(PANO_PROJECTION_MAX_PARMS);
@@ -314,6 +327,7 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
                         5);       // border width
         m_projParamTextCtrl[i] = new wxTextCtrl(this, PROJ_PARAM_VAL_ID+i, wxT("0"),
                                     wxDefaultPosition, wxSize(35,-1), wxTE_PROCESS_ENTER);
+        m_projParamTextCtrl[i]->PushEventHandler(new TextKillFocusHandler(this));
         m_projParamSizer->Add(m_projParamTextCtrl[i],
                         0,        // not vertically strechable
                         wxALL | wxALIGN_CENTER_VERTICAL, // draw border all around
@@ -322,7 +336,7 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
         m_projParamSlider[i] = new wxSlider(this, PROJ_PARAM_SLIDER_ID+i, 0, -90, 90);
         m_projParamSizer->Add(m_projParamSlider[i],
                         1,        // not vertically strechable
-                        wxALL | wxALIGN_CENTER_VERTICAL | wxEXPAND, // draw border all around
+                        wxALL | wxALIGN_CENTER_VERTICAL, // draw border all around
                         5);       // border width
     }
 
@@ -374,7 +388,11 @@ PreviewFrame::PreviewFrame(wxFrame * frame, PT::Panorama &pano)
     }
     SetStatusText(_("Center panorama with left mouse button, set horizon with right button"),0);
     wxAcceleratorEntry entries[3];
+#ifdef __WXMAC
+    entries[0].Set(wxACCEL_CMD,(int)'F',ID_FULL_SCREEN);
+#else
     entries[0].Set(wxACCEL_NORMAL,WXK_F11,ID_FULL_SCREEN);
+#endif
     entries[1].Set(wxACCEL_CMD,(int)'Z',ID_UNDO);
     entries[2].Set(wxACCEL_CMD,(int)'R',ID_REDO);
     wxAcceleratorTable accel(3, entries);
@@ -402,6 +420,10 @@ PreviewFrame::~PreviewFrame()
     bool checked = m_ToolBar->GetToolState(XRCID("preview_auto_update_tool"));
     config->Write(wxT("/PreviewFrame/autoUpdate"), checked ? 1l: 0l);
     config->Write(wxT("/PreviewFrame/blendMode"), m_BlendModeChoice->GetSelection());
+    for (int i=0; i < PANO_PROJECTION_MAX_PARMS; i++)
+    {
+        m_projParamTextCtrl[i]->PopEventHandler(true);
+    };
     m_pano.removeObserver(this);
     DEBUG_TRACE("dtor end");
 }
@@ -410,7 +432,7 @@ void PreviewFrame::OnChangeDisplayedImgs(wxCommandEvent & e)
 {
     int id = e.GetId() - ID_TOGGLE_BUT;
     int nImg = m_pano.getNrOfImages();
-    UIntSet activeImages = m_pano.getActiveImages();
+    HuginBase::UIntSet activeImages = m_pano.getActiveImages();
     DEBUG_DEBUG("toggle_button_id: " << id << " nImg:" << nImg << "  m_ToggleButtons.size(): " << m_ToggleButtons.size());
     if (id >= 0 && id < nImg) {
         if (e.IsChecked()) {
@@ -419,24 +441,24 @@ void PreviewFrame::OnChangeDisplayedImgs(wxCommandEvent & e)
             activeImages.erase(id);
         }
 //        m_PreviewPanel->SetDisplayedImages(m_displayedImgs);
-    	GlobalCmdHist::getInstance().addCommand(
-            new PT::SetActiveImagesCmd(m_pano, activeImages)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetActiveImagesCmd(m_pano, activeImages)
         );
     } else {
         DEBUG_ERROR("invalid Togglebutton ID");
     }
 }
 
-void PreviewFrame::panoramaChanged(Panorama &pano)
+void PreviewFrame::panoramaChanged(HuginBase::Panorama &pano)
 {
-    const PanoramaOptions & opts = pano.getOptions();
+    const HuginBase::PanoramaOptions & opts = pano.getOptions();
 
     wxString projection;
     m_ProjectionChoice->SetSelection(opts.getProjection());
     m_VFOVSlider->Enable( opts.fovCalcSupported(opts.getProjection()) );
 
     m_outputModeChoice->SetSelection(opts.outputMode);
-    if (opts.outputMode == PanoramaOptions::OUTPUT_HDR) {
+    if (opts.outputMode == HuginBase::PanoramaOptions::OUTPUT_HDR) {
         /*
         m_exposureTextCtrl->Hide();
         m_defaultExposureBut->Hide();
@@ -451,7 +473,7 @@ void PreviewFrame::panoramaChanged(Panorama &pano)
         m_incExposureBut->Show();
         */
     }
-    m_exposureTextCtrl->SetValue(wxString(doubleToString(opts.outputExposureValue,2).c_str(), wxConvLocal));
+    m_exposureTextCtrl->SetValue(wxString(hugin_utils::doubleToString(opts.outputExposureValue,2).c_str(), wxConvLocal));
 
     bool activeImgs = pano.getActiveImages().size() > 0;
     m_ToolBar->EnableTool(XRCID("preview_center_tool"), activeImgs);
@@ -475,7 +497,7 @@ void PreviewFrame::panoramaChanged(Panorama &pano)
                 wxString str2(pp->name, wxConvLocal);
                 str2 = wxGetTranslation(str2);
                 m_projParamNamesLabel[i]->SetLabel(str2);
-                m_projParamSlider[i]->SetRange(roundi(pp->minValue), roundi(pp->maxValue));
+                m_projParamSlider[i]->SetRange(hugin_utils::roundi(pp->minValue), hugin_utils::roundi(pp->maxValue));
             }
             for(;i < PANO_PROJECTION_MAX_PARMS; i++) {
                 m_projParamNamesLabel[i]->Hide();
@@ -493,24 +515,25 @@ void PreviewFrame::panoramaChanged(Panorama &pano)
         std::vector<double> params = opts.getProjectionParameters();
         assert((int) params.size() == nParam);
         for (int i=0; i < nParam; i++) {
-            wxString val = wxString(doubleToString(params[i],1).c_str(), wxConvLocal);
+            wxString val = wxString(hugin_utils::doubleToString(params[i],1).c_str(), wxConvLocal);
             m_projParamTextCtrl[i]->SetValue(wxString(val.wc_str(), wxConvLocal));
-            m_projParamSlider[i]->SetValue(roundi(params[i]));
+            m_projParamSlider[i]->SetValue(hugin_utils::roundi(params[i]));
         }
     }
     if (relayout) {
         m_topsizer->Layout();
+        Refresh();
     }
     SetStatusText(_("Center panorama with left mouse button, set horizon with right button"),0);
     SetStatusText(wxString::Format(wxT("%.1f x %.1f"), opts.getHFOV(), opts.getVFOV()),2);
-    m_HFOVSlider->SetValue(roundi(opts.getHFOV()));
-    m_VFOVSlider->SetValue(roundi(opts.getVFOV()));
+    m_HFOVSlider->SetValue(hugin_utils::roundi(opts.getHFOV()));
+    m_VFOVSlider->SetValue(hugin_utils::roundi(opts.getVFOV()));
 
     m_oldProjFormat = opts.getProjection();
     updatePano();
 }
 
-void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
+void PreviewFrame::panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet &changed)
 {
     DEBUG_TRACE("");
 
@@ -532,9 +555,8 @@ void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
 
     // add buttons
     if ( nrImages >= nrButtons ) {
-        for(UIntSet::const_iterator it = changed.begin(); it != changed.end(); ++it){
+        for(HuginBase::UIntSet::const_iterator it = changed.begin(); it != changed.end(); ++it){
             if (*it >= nrButtons) {
-                unsigned int imgNr = *it;
                 // create new item.
 //                wxImage * bmp = new wxImage(sz.GetWidth(), sz.GetHeight());
 #ifdef USE_TOGGLE_BUTTON
@@ -557,7 +579,7 @@ void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
                 but->SetValue(true);
                 m_ButtonSizer->Add(but,
                                    0,
-                                   wxLEFT | wxTOP | wxADJUST_MINSIZE,
+                                   wxLEFT | wxTOP,
                                    5);
                 m_ToggleButtons.push_back(but);
                 dirty = true;
@@ -566,7 +588,7 @@ void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
     }
 
     // update existing items
-    UIntSet displayedImages = m_pano.getActiveImages();
+    HuginBase::UIntSet displayedImages = m_pano.getActiveImages();
     for (unsigned i=0; i < nrImages; i++) {
         m_ToggleButtons[i]->SetValue(set_contains(displayedImages, i));
         wxFileName tFilename(wxString (pano.getImage(i).getFilename().c_str(), HUGIN_CONV_FILENAME));
@@ -574,7 +596,11 @@ void PreviewFrame::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
     }
 
     if (dirty) {
+#if wxCHECK_VERSION(3,0,0)
+        m_ButtonSizer->FitInside(m_ButtonPanel);
+#else
 		m_ButtonSizer->SetVirtualSizeHints(m_ButtonPanel);
+#endif
 		// Layout();
 		DEBUG_INFO("New m_ButtonPanel width: " << (m_ButtonPanel->GetSize()).GetWidth());
 		DEBUG_INFO("New m_ButtonPanel Height: " << (m_ButtonPanel->GetSize()).GetHeight());
@@ -618,8 +644,8 @@ void PreviewFrame::OnProjectionChanged()
     case PanoramaOptions::EQUIRECTANGULAR:   Ip = _("Equirectangular"); break;
     }
     opt.projectionFormat = (PanoramaOptions::ProjectionFormat) lt;
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( pano, opt )
         );
     DEBUG_DEBUG ("Projection changed: "  << lt << ":" << Ip )
 
@@ -631,8 +657,8 @@ void PreviewFrame::OnCenterHorizontally(wxCommandEvent & e)
 {
     if (m_pano.getActiveImages().size() == 0) return;
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::CenterPanoCmd(m_pano)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::CenterPanoCmd(m_pano)
         );
 }
 
@@ -640,8 +666,8 @@ void PreviewFrame::OnStraighten(wxCommandEvent & e)
 {
     if (m_pano.getNrOfImages() == 0) return;
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::StraightenPanoCmd(m_pano)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::StraightenPanoCmd(m_pano)
         );
 }
 
@@ -662,15 +688,14 @@ void PreviewFrame::OnFitPano(wxCommandEvent & e)
     if (m_pano.getActiveImages().size() == 0) return;
 
     DEBUG_TRACE("");
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::CalculateFitPanorama fitPano(m_pano);
+    fitPano.run();
+    opt.setHFOV(fitPano.getResultHorizontalFOV());
+    opt.setHeight(hugin_utils::roundi(fitPano.getResultHeight()));
 
-    double hfov, height;
-    m_pano.fitPano(hfov, height);
-    opt.setHFOV(hfov);
-    opt.setHeight(roundi(height));
-
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
         );
 
     DEBUG_INFO ( "new fov: [" << opt.getHFOV() << " "<< opt.getVFOV() << "] => height: " << opt.getHeight() );
@@ -682,12 +707,12 @@ void PreviewFrame::OnShowAll(wxCommandEvent & e)
     if (m_pano.getNrOfImages() == 0) return;
 
     DEBUG_ASSERT(m_pano.getNrOfImages() == m_ToggleButtons.size());
-    UIntSet displayedImgs;
+    HuginBase::UIntSet displayedImgs;
     for (unsigned int i=0; i < m_pano.getNrOfImages(); i++) {
         displayedImgs.insert(i);
     }
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetActiveImagesCmd(m_pano, displayedImgs)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetActiveImagesCmd(m_pano, displayedImgs)
         );
     updatePano();
 }
@@ -700,9 +725,9 @@ void PreviewFrame::OnShowNone(wxCommandEvent & e)
     for (unsigned int i=0; i < m_pano.getNrOfImages(); i++) {
         m_ToggleButtons[i]->SetValue(false);
     }
-    UIntSet displayedImgs;
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetActiveImagesCmd(m_pano, displayedImgs)
+    HuginBase::UIntSet displayedImgs;
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetActiveImagesCmd(m_pano, displayedImgs)
         );
     updatePano();
 }
@@ -717,24 +742,24 @@ void PreviewFrame::OnNumTransform(wxCommandEvent & e)
     if (dlg.ShowModal() == wxID_OK ) {
         wxString text = XRCCTRL(dlg, "numtrans_yaw", wxTextCtrl)->GetValue();
         double y;
-        if (!str2double(text, y)) {
+        if (!hugin_utils::str2double(text, y)) {
             wxLogError(_("Yaw value must be numeric."));
             return;
         }
         text = XRCCTRL(dlg, "numtrans_pitch", wxTextCtrl)->GetValue();
         double p;
-        if (!str2double(text, p)) {
+        if (!hugin_utils::str2double(text, p)) {
             wxLogError(_("Pitch value must be numeric."));
             return;
         }
         text = XRCCTRL(dlg, "numtrans_roll", wxTextCtrl)->GetValue();
         double r;
-        if (!str2double(text, r)) {
+        if (!hugin_utils::str2double(text, r)) {
             wxLogError(_("Roll value must be numeric."));
             return;
         }
-        GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(m_pano, y, p, r)
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::RotatePanoCmd(m_pano, y, p, r)
             );
         // update preview panel
         updatePano();
@@ -745,14 +770,14 @@ void PreviewFrame::OnNumTransform(wxCommandEvent & e)
 
 void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
 {
-    PanoramaOptions opts = m_pano.getOptions();
+    HuginBase::PanoramaOptions opts = m_pano.getOptions();
     if (e.GetEventObject() == m_exposureTextCtrl) {
         // exposure
         wxString text = m_exposureTextCtrl->GetValue();
         DEBUG_INFO ("target exposure = " << text.mb_str(wxConvLocal) );
         double p = 0;
         if (text != wxT("")) {
-            if (!str2double(text, p)) {
+            if (!hugin_utils::str2double(text, p)) {
                 wxLogError(_("Value must be numeric."));
                 return;
             }
@@ -767,7 +792,7 @@ void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
                 DEBUG_INFO ("param " << i << ":  = " << text.mb_str(wxConvLocal) );
                 double p = 0;
                 if (text != wxT("")) {
-                    if (!str2double(text, p)) {
+                    if (!hugin_utils::str2double(text, p)) {
                         wxLogError(_("Value must be numeric."));
                         return;
                     }
@@ -777,8 +802,8 @@ void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
         }
         opts.setProjectionParameters(para);
     }
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( m_pano, opts )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( m_pano, opts )
                                            );
     // update preview panel
     updatePano();
@@ -787,10 +812,10 @@ void PreviewFrame::OnTextCtrlChanged(wxCommandEvent & e)
 
 void PreviewFrame::OnProjParameterReset(wxCommandEvent &e)
 {
-    PanoramaOptions opts=m_pano.getOptions();
+    HuginBase::PanoramaOptions opts = m_pano.getOptions();
     opts.resetProjectionParameters();
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd(m_pano, opts)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd(m_pano, opts)
         );
 };
 
@@ -798,7 +823,7 @@ void PreviewFrame::OnChangeFOV(wxScrollEvent & e)
 {
     DEBUG_TRACE("");
 
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
 
     if (e.GetEventObject() == m_HFOVSlider) {
         DEBUG_DEBUG("HFOV changed (slider): " << e.GetInt() << " == " << m_HFOVSlider->GetValue());
@@ -820,8 +845,8 @@ void PreviewFrame::OnChangeFOV(wxScrollEvent & e)
 		opt.setVFOV(m_VFOVSlider->GetValue());
     }
 
-    GlobalCmdHist::getInstance().addCommand(
-        new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+        new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
         );
 }
 
@@ -848,10 +873,10 @@ void PreviewFrame::OnBlendChoice(wxCommandEvent & e)
 void PreviewFrame::OnDefaultExposure( wxCommandEvent & e )
 {
     if (m_pano.getNrOfImages() > 0) {
-        PanoramaOptions opt = m_pano.getOptions();
-        opt.outputExposureValue = calcMeanExposure(m_pano);
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( m_pano, opt )
+        HuginBase::PanoramaOptions opt = m_pano.getOptions();
+        opt.outputExposureValue = HuginBase::CalculateMeanExposure::calcMeanExposure(m_pano);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                                );
         updatePano();
     }
@@ -859,20 +884,20 @@ void PreviewFrame::OnDefaultExposure( wxCommandEvent & e )
 
 void PreviewFrame::OnIncreaseExposure( wxSpinEvent & e )
 {
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
     opt.outputExposureValue = opt.outputExposureValue + 1.0/3;
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                             );
     updatePano();
 }
 
 void PreviewFrame::OnDecreaseExposure( wxSpinEvent & e )
 {
-    PanoramaOptions opt = m_pano.getOptions();
+    HuginBase::PanoramaOptions opt = m_pano.getOptions();
     opt.outputExposureValue = opt.outputExposureValue - 1.0/3;
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::SetPanoOptionsCmd( m_pano, opt )
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                            );
     updatePano();
 }
@@ -880,12 +905,12 @@ void PreviewFrame::OnDecreaseExposure( wxSpinEvent & e )
 void PreviewFrame::OnProjectionChoice( wxCommandEvent & e )
 {
     if (e.GetEventObject() == m_ProjectionChoice) {
-        PanoramaOptions opt = m_pano.getOptions();
+        HuginBase::PanoramaOptions opt = m_pano.getOptions();
         int lt = m_ProjectionChoice->GetSelection();
         wxString Ip;
-        opt.setProjection( (PanoramaOptions::ProjectionFormat) lt );
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( m_pano, opt )
+        opt.setProjection((HuginBase::PanoramaOptions::ProjectionFormat) lt);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                             );
         DEBUG_DEBUG ("Projection changed: "  << lt);
         updatePano();
@@ -898,12 +923,12 @@ void PreviewFrame::OnProjectionChoice( wxCommandEvent & e )
 void PreviewFrame::OnOutputChoice( wxCommandEvent & e)
 {
     if (e.GetEventObject() == m_outputModeChoice) {
-        PanoramaOptions opt = m_pano.getOptions();
+        HuginBase::PanoramaOptions opt = m_pano.getOptions();
         int lt = m_outputModeChoice->GetSelection();
         wxString Ip;
-        opt.outputMode = ( (PanoramaOptions::OutputMode) lt );
-        GlobalCmdHist::getInstance().addCommand(
-                new PT::SetPanoOptionsCmd( m_pano, opt )
+        opt.outputMode = ((HuginBase::PanoramaOptions::OutputMode) lt);
+        PanoCommand::GlobalCmdHist::getInstance().addCommand(
+                new PanoCommand::SetPanoOptionsCmd( m_pano, opt )
                                                );
         updatePano();
 
@@ -916,38 +941,19 @@ void PreviewFrame::OnOutputChoice( wxCommandEvent & e)
 void PreviewFrame::updateProgressDisplay()
 {
     wxString msg;
-    // build the message:
-    for (std::vector<AppBase::ProgressTask>::iterator it = tasks.begin();
-         it != tasks.end(); ++it)
+    if (!m_message.empty())
     {
-        wxString cMsg;
-        if (it->getProgress() > 0) {
-            cMsg.Printf(wxT("%s [%3.0f%%]: %s "),
-                        wxString(it->getShortMessage().c_str(), wxConvLocal).c_str(),
-                        100 * it->getProgress(),
-                        wxString(it->getMessage().c_str(), wxConvLocal).c_str());
-        } else {
-            cMsg.Printf(wxT("%s %s"),wxString(it->getShortMessage().c_str(), wxConvLocal).c_str(),
-                        wxString(it->getMessage().c_str(), wxConvLocal).c_str());
-        }
-        // append to main message
-        if (it == tasks.begin()) {
-            msg = cMsg;
-        } else {
-            msg.Append(wxT(" | "));
-            msg.Append(cMsg);
-        }
-    }
-//    wxStatusBar *m_statbar = GetStatusBar();
-    //DEBUG_TRACE("Statusmb : " << msg.mb_str(wxConvLocal));
-    //m_statbar->SetStatusText(msg,0);
+        msg = wxGetTranslation(wxString(m_message.c_str(), wxConvLocal));
+        if (!m_filename.empty())
+        {
+            msg.Append(wxT(" "));
+            msg.Append(wxString(ProgressDisplay::m_filename.c_str(), HUGIN_CONV_FILENAME));
+        };
+    };
+    GetStatusBar()->SetStatusText(msg, 0);
 
 #ifdef __WXMSW__
     UpdateWindow(NULL);
-#else
-    // This is a bad call.. we just want to repaint the window, instead we will
-    // process user events as well :( Unfortunately, there is not portable workaround...
-//    wxYield();
 #endif
 }
 
@@ -963,7 +969,7 @@ void PreviewFrame::OnFullScreen(wxCommandEvent & e)
 
 void PreviewFrame::OnUndo(wxCommandEvent &e)
 {
-    if(GlobalCmdHist::getInstance().canUndo())
+    if(PanoCommand::GlobalCmdHist::getInstance().canUndo())
     {
         wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITUNDO"));
         m_parent->GetEventHandler()->AddPendingEvent(dummy);
@@ -976,7 +982,7 @@ void PreviewFrame::OnUndo(wxCommandEvent &e)
 
 void PreviewFrame::OnRedo(wxCommandEvent &e)
 {
-    if(GlobalCmdHist::getInstance().canRedo())
+    if(PanoCommand::GlobalCmdHist::getInstance().canRedo())
     {
         wxCommandEvent dummy(wxEVT_COMMAND_MENU_SELECTED, XRCID("ID_EDITREDO"));
         m_parent->GetEventHandler()->AddPendingEvent(dummy);
