@@ -14,8 +14,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -32,9 +32,6 @@
 #include <wx/platform.h>
 
 //multitexture feature requires glew on some systems
-#if !defined Hugin_shared || !defined _WINDOWS
-#define GLEW_STATIC
-#endif
 #include <GL/glew.h>
 
 #ifdef __WXMAC__
@@ -62,6 +59,42 @@
 bool PreviewIdentifyTool::texture_created = false;
 unsigned int PreviewIdentifyTool::circle_border_tex;
 unsigned int PreviewIdentifyTool::rectangle_border_tex;
+unsigned int PreviewIdentifyTool::font_tex;
+unsigned int PreviewIdentifyTool::font_list;
+std::vector<int> PreviewIdentifyTool::m_glyphWidth;
+
+#define FONT_TEXTURE_HEIGHT 75
+wxBitmap GenerateFontTexture(const int textureHeight, int& textureWidth, std::vector<int>& glyphWidth)
+{
+    wxBitmap bitmap(10 * textureHeight, textureHeight);
+    wxMemoryDC dc(bitmap);
+    dc.SetBackground(*wxBLACK_BRUSH);
+    dc.Clear();
+#if wxCHECK_VERSION(3,0,0)
+    wxFont font(wxSize(0, textureHeight), wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD);
+    dc.SetFont(font);
+#else
+    wxFont* font=wxFont::New(wxSize(0, textureHeight), wxFONTFAMILY_DEFAULT, wxFONTSTYLE_NORMAL, wxFONTWEIGHT_BOLD, false, wxT(""));
+    dc.SetFont(*font);
+#endif    
+    dc.SetTextForeground(*wxWHITE);
+    dc.DrawText(wxT("0123456789"), 0, 0);
+    textureWidth = 0;
+    glyphWidth.resize(10, 0);
+    for (int i = 0; i < 10; ++i)
+    {
+        wxString number;
+        number << i;
+        wxSize textSize = dc.GetTextExtent(number);
+        textureWidth += textSize.GetWidth();
+        glyphWidth[i] = textSize.GetWidth();
+    };
+#if !wxCHECK_VERSION(3,0,0)
+    delete font;
+#endif  
+    dc.SelectObject(wxNullBitmap);
+    return bitmap;
+}
 
 PreviewIdentifyTool::PreviewIdentifyTool(ToolHelper *helper,
                                          GLPreviewFrame *owner)
@@ -170,6 +203,50 @@ PreviewIdentifyTool::PreviewIdentifyTool(ToolHelper *helper,
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
         }
+        {
+            int textureWidth;
+            wxBitmap fontTexture(GenerateFontTexture(FONT_TEXTURE_HEIGHT, textureWidth, m_glyphWidth));
+            wxImage image = fontTexture.ConvertToImage();
+            glGenTextures(1, (GLuint*)&font_tex);
+            glBindTexture(GL_TEXTURE_2D, font_tex);
+            unsigned char* font_tex_data = new unsigned char[FONT_TEXTURE_HEIGHT * textureWidth * 2];
+            for (size_t x = 0; x < textureWidth; ++x)
+            {
+                for (size_t y = 0; y < FONT_TEXTURE_HEIGHT; ++y)
+                {
+                    const unsigned char value = image.GetRed(x, y);
+                    font_tex_data[2 * y * textureWidth + 2 * x] = value;
+                    font_tex_data[2 * y * textureWidth + 2 * x + 1] = value;
+                };
+            };
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
+            gluBuild2DMipmaps(GL_TEXTURE_2D, GL_LUMINANCE_ALPHA, textureWidth, FONT_TEXTURE_HEIGHT,
+                GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, font_tex_data);
+            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
+                GL_LINEAR_MIPMAP_LINEAR);
+            delete[] font_tex_data;
+
+            font_list = glGenLists(10);
+            float posX = 0;
+            for (int loop = 0; loop<10; loop++)
+            {
+                glNewList(font_list + loop, GL_COMPILE);
+                glBegin(GL_QUADS);
+                glTexCoord2f(posX, 0);
+                glVertex2i(0, 0);
+                glTexCoord2f(posX + m_glyphWidth[loop] / static_cast<float>(textureWidth), 0);
+                glVertex2i(m_glyphWidth[loop], 0);
+                glTexCoord2f(posX + m_glyphWidth[loop] / static_cast<float>(textureWidth), 1);
+                glVertex2i(m_glyphWidth[loop], FONT_TEXTURE_HEIGHT);
+                glTexCoord2f(posX, 1);
+                glVertex2i(0, FONT_TEXTURE_HEIGHT);
+                glEnd();
+                glTranslatef(m_glyphWidth[loop], 0, 0);
+                glEndList();
+                posX += m_glyphWidth[loop] / static_cast<float>(textureWidth);
+            }
+        }
         texture_created = true;
     }
 }
@@ -178,6 +255,8 @@ PreviewIdentifyTool::~PreviewIdentifyTool()
     // free the textures
     glDeleteTextures(1, (GLuint*) &rectangle_border_tex);
     glDeleteTextures(1, (GLuint*) &circle_border_tex);
+    glDeleteLists(font_list, 10);
+    glDeleteTextures(1, (GLuint*) &font_tex);
 }
 
 void PreviewIdentifyTool::Activate()
@@ -197,14 +276,15 @@ void PreviewIdentifyTool::Activate()
     /* TODO if it becomes possible to activate the tool by a keyboard shortcut
      * or something, call ImagesUnderMouseChangedEvent() to make sure we display
      * indicators for images currently under the cursor. */
-     
-     helper->SetStatusMessage(_("Move the mouse over the images or image buttons to identify them."));
+    ImagesUnderMouseChangedEvent();
+
+    helper->SetStatusMessage(_("Move the mouse over the images or image buttons to identify them."));
 }
 
 void PreviewIdentifyTool::StopUpdating() {
-    if (image_set.size() > 0) {
+    if (!image_set.empty()) {
         std::set<unsigned int>::iterator iterator;
-        for (iterator = image_set.begin(); iterator != image_set.end(); iterator++)
+        for (iterator = image_set.begin(); iterator != image_set.end(); ++iterator)
         {
             DEBUG_ASSERT(*iterator < helper->GetPanoramaPtr()->getNrOfImages());
             // reset this button to its default system colour.
@@ -214,6 +294,7 @@ void PreviewIdentifyTool::StopUpdating() {
         }
     }
     image_set.clear();
+    preview_frame->UpdateIdentifyTools(image_set);
     stopUpdating = true;
     ForceRedraw();
 }
@@ -271,10 +352,10 @@ void PreviewIdentifyTool::KeypressEvent(int keycode, int modifierss, int pressed
             holdControl = true;
             ContinueUpdating();
         } else {
-            holdControl = false;
-            if (!constantOn) {
+            if(holdControl) {
                 StopUpdating();
             }
+            holdControl = false;
         }
     }
 }
@@ -328,7 +409,7 @@ void PreviewIdentifyTool::AfterDrawImagesEvent()
     // on top. We will folow this convention to avoid confusion.
     glMatrixMode(GL_MODELVIEW);
     std::set<unsigned int>::reverse_iterator it;
-    for (it = image_set.rbegin(); it != image_set.rend(); it++)
+    for (it = image_set.rbegin(); it != image_set.rend(); ++it)
     {
         DEBUG_ASSERT(*it < helper->GetPanoramaPtr()->getNrOfImages());
         helper->GetViewStatePtr()->GetTextureManager()->
@@ -341,7 +422,9 @@ void PreviewIdentifyTool::AfterDrawImagesEvent()
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     unsigned int image_counter = 0;
-    for (it = image_set.rbegin(); it != image_set.rend(); it++)
+    const unsigned int canvasWidth = helper->GetViewStatePtr()->GetOptions()->getWidth();
+    const unsigned int canvasHeight = helper->GetViewStatePtr()->GetOptions()->getHeight();
+    for (it = image_set.rbegin(); it != image_set.rend(); ++it)
     {
         glMatrixMode(GL_TEXTURE);
         // Use the mask to alter the shape of the identification boxes, but
@@ -412,6 +495,50 @@ void PreviewIdentifyTool::AfterDrawImagesEvent()
         glPopMatrix();
         // tell the preview frame to update the button to show the same colour.
         preview_frame->SetImageButtonColour(*it, r, g, b);
+        // draw number
+        HuginBase::PanoramaOptions* opts = helper->GetViewStatePtr()->GetOptions();
+        HuginBase::SrcPanoImage* img = helper->GetViewStatePtr()->GetSrcImage(*it);
+        HuginBase::PTools::Transform transform;
+        transform.createInvTransform(*img, *opts);
+        hugin_utils::FDiff2D imageCenter(crop_region.upperLeft() + crop_region.size() / 2);
+        hugin_utils::FDiff2D imageCenterPano;
+        if (transform.transformImgCoord(imageCenterPano, imageCenter) && !wxGetKeyState(WXK_ALT))
+        {
+            glBindTexture(GL_TEXTURE_2D, font_tex);
+            if (helper->GetViewStatePtr()->GetSupportMultiTexture())
+            {
+                glActiveTexture(GL_TEXTURE1);
+                glDisable(GL_TEXTURE_2D);
+                glActiveTexture(GL_TEXTURE0);
+            };
+            glMatrixMode(GL_MODELVIEW);
+            glPushMatrix();
+            wxString number;
+            number << *it;
+            int textWidth = 0;
+            unsigned char* listIndex = new unsigned char[number.Length()];
+            for (size_t i = 0; i < number.Length(); ++i)
+            {
+#if wxCHECK_VERSION(3,0,0)
+                textWidth += m_glyphWidth[number[i].GetValue() - 48];
+                listIndex[i] = number[i].GetValue() - 48;
+#else
+                textWidth += m_glyphWidth[number[i] - 48];
+                listIndex[i] = number[i] - 48;
+#endif
+            }
+            float scaleFactor = std::min(canvasWidth, canvasHeight) / static_cast<float>(FONT_TEXTURE_HEIGHT) / 10;
+            imageCenterPano.x = std::min<double>(std::max<double>(imageCenterPano.x, textWidth*scaleFactor / 2.0), canvasWidth - textWidth * scaleFactor / 2.0);
+            imageCenterPano.y = std::min<double>(std::max<double>(imageCenterPano.y, FONT_TEXTURE_HEIGHT*scaleFactor / 2.0), canvasHeight - FONT_TEXTURE_HEIGHT * scaleFactor / 2.0);
+            glTranslatef(imageCenterPano.x, imageCenterPano.y, 0);
+            glScalef(scaleFactor, scaleFactor, 1.0);
+            glTranslatef(-textWidth / 2, -FONT_TEXTURE_HEIGHT / 2, 0);
+            glListBase(font_list);
+            glCallLists(number.Length(), GL_UNSIGNED_BYTE, listIndex);
+            glMatrixMode(GL_MODELVIEW);
+            glPopMatrix();
+            delete[] listIndex;
+        };
     }
     // set stuff back how we found it.
     glMatrixMode(GL_MODELVIEW);
@@ -511,14 +638,23 @@ void PreviewIdentifyTool::MouseButtonEvent(wxMouseEvent & e)
         holdLeft = true;
     } 
 
-    if (holdLeft && e.LeftUp() && image_set.size() == 2) {
+    if (holdLeft && e.LeftUp() && (image_set.size()==1 || image_set.size() == 2)) 
+    {
         holdLeft = false;
-        if (constantOn) {
-                // when there are only two images with indicators shown, show the
-                // control point editor with those images when left clicked.
+        if (constantOn || e.CmdDown())
+        {
+            // when there are only two images with indicators shown, show the
+            // control point editor with those images when left clicked.
+            if(image_set.size()==2)
+            {
                 MainFrame::Get()->ShowCtrlPointEditor(*(image_set.begin()),
-                                                      *(++image_set.begin()));
-                MainFrame::Get()->Raise();
+                                                        *(++image_set.begin()));
+            }
+            else
+            {
+                MainFrame::Get()->ShowMaskEditor(*image_set.begin());
+            };
+            MainFrame::Get()->Raise();
         }
     }
 
@@ -540,18 +676,15 @@ void PreviewIdentifyTool::UpdateWithNewImageSet(std::set<unsigned int> new_image
     // If we are currently showing indicators for some of the images, we want
     // to work out which ones are not in the new set, so we can set their
     // buttons back to the system colour.
-    std::vector<unsigned int>::iterator end;
+    if (!image_set.empty())
     {
-        std::vector<unsigned int> difference(image_set.size());
-        end = difference.begin();
-        end = std::set_difference (image_set.begin(), image_set.end(),
-                                   new_image_set.begin(), new_image_set.end(),
-                                   difference.begin());
-        DEBUG_ASSERT(end >= difference.begin() && end <= difference.end());
-        if (difference.begin() != end)
+        HuginBase::UIntSet difference;
+        std::set_difference (image_set.begin(), image_set.end(),
+                             new_image_set.begin(), new_image_set.end(),
+                             std::inserter(difference,difference.end()));
+        if (!difference.empty())
         {
-            std::vector<unsigned int>::iterator iterator;
-            for (iterator = difference.begin(); iterator != end; iterator++)
+            for (HuginBase::UIntSet::iterator iterator = difference.begin(); iterator != difference.end(); ++iterator)
             {
                 DEBUG_ASSERT(*iterator < helper->GetPanoramaPtr()->getNrOfImages());
                 // reset this button to its default system colour.
@@ -563,22 +696,22 @@ void PreviewIdentifyTool::UpdateWithNewImageSet(std::set<unsigned int> new_image
     }
 
     // now request to be notified when drawing the new ones.
-    std::vector<unsigned int> difference(new_image_set.size());
-    end = difference.begin();
-    end = std::set_difference (new_image_set.begin(), new_image_set.end(),
-                               image_set.begin(), image_set.end(),
-                               difference.begin());
-    DEBUG_ASSERT(end >= difference.begin() && end <= difference.end());
-    if (difference.begin() != end)
+    if (!new_image_set.empty())
     {
-        std::vector<unsigned int>::iterator iterator;
-        for (iterator = difference.begin(); iterator != end; iterator++)
+        HuginBase::UIntSet difference;
+        std::set_difference(new_image_set.begin(), new_image_set.end(),
+            image_set.begin(), image_set.end(),
+            std::inserter(difference, difference.end()));
+        if (!difference.empty())
         {
-            DEBUG_ASSERT(*iterator < helper->GetPanoramaPtr()->getNrOfImages());
-            // get notification of when this is about to be drawn.
-            helper->NotifyMeBeforeDrawing(*iterator, this);
+            for (HuginBase::UIntSet::iterator iterator = difference.begin(); iterator != difference.end(); ++iterator)
+            {
+                DEBUG_ASSERT(*iterator < helper->GetPanoramaPtr()->getNrOfImages());
+                // get notification of when this is about to be drawn.
+                helper->NotifyMeBeforeDrawing(*iterator, this);
+            }
         }
-    }
+    };
     // remember the new set.
     image_set.swap(new_image_set);
 
