@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,34 +30,23 @@
 #include "panoinc.h"
 
 #include <vigra/basicimageview.hxx>
-#include "vigra_ext/blend.h"
-#include "PT/Stitcher.h"
-
+#include "nona/Stitcher.h"
 
 #include "base_wx/wxImageCache.h"
 #include "hugin/PreviewPanel.h"
 #include "hugin/PreviewFrame.h"
 #include "hugin/MainFrame.h"
-#include "hugin/CommandHistory.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/PanoCommand.h"
+#include "base_wx/wxcms.h"
 #include "hugin/config_defaults.h"
 #include "hugin/huginApp.h"
-//#include "hugin/ImageProcessing.h"
 
 #include <math.h>
 
-using namespace PT;
-using namespace std;
-using namespace vigra;
-using namespace vigra_ext;
-using namespace hugin_utils;
-
-typedef RGBValue<unsigned char> BRGBValue;
+typedef vigra::RGBValue<unsigned char> BRGBValue;
 
 BEGIN_EVENT_TABLE(PreviewPanel, wxPanel)
-//    EVT_PAINT(CPImageCtrl::OnPaint)
-//    EVT_LEFT_DOWN(CPImageCtrl::mousePressEvent)
-//    EVT_MOTION(CPImageCtrl::mouseMoveEvent)
-//    EVT_LEFT_UP(CPImageCtrl::mouseReleaseEvent)
     EVT_SIZE(PreviewPanel::OnResize)
     EVT_LEFT_DOWN(PreviewPanel::mousePressLMBEvent)
     EVT_RIGHT_DOWN(PreviewPanel::mousePressRMBEvent)
@@ -70,7 +59,6 @@ PreviewPanel::PreviewPanel()
     m_panoBitmap(0), 
     m_pano2erect(0), m_blendMode(BLEND_COPY), 
     m_state_rendering(false), m_rerender(false), m_imgsDirty(true)
-
 {
 }
 
@@ -98,7 +86,7 @@ bool PreviewPanel::Create(wxWindow* parent, wxWindowID id,
 }
 
 
-void PreviewPanel::Init(PreviewFrame *parent, PT::Panorama * panorama )
+void PreviewPanel::Init(PreviewFrame *parent, HuginBase::Panorama * panorama )
 {
     pano = panorama;
     parentWindow = parent;
@@ -117,17 +105,16 @@ PreviewPanel::~PreviewPanel()
     if (m_pano2erect) {
         delete m_pano2erect;
     }
-	delete & vigra_ext::ThreadManager::get();
     DEBUG_TRACE("dtor end");
 }
 
-void PreviewPanel::panoramaChanged(Panorama &pano)
+void PreviewPanel::panoramaChanged(HuginBase::Panorama &pano)
 {
     // avoid recursive calls.. don't know if they can happen at all,
     // but they might lead to crashes.
     bool dirty = false;
 
-    const PanoramaOptions & newOpts = pano.getOptions();
+    const HuginBase::PanoramaOptions & newOpts = pano.getOptions();
 
     // check if an important options has been changed
     if (newOpts.getHFOV() != opts.getHFOV()) {
@@ -170,40 +157,12 @@ void PreviewPanel::panoramaChanged(Panorama &pano)
     }
 }
 
-void PreviewPanel::panoramaImagesChanged(Panorama &pano, const UIntSet &changed)
+void PreviewPanel::panoramaImagesChanged(HuginBase::Panorama &pano, const HuginBase::UIntSet &changed)
 {
     DEBUG_TRACE("");
-
-    /*
-    for(PT::UIntSet::const_iterator it = changed.begin(); it != changed.end();
-        ++it)
-    {
-        // TODO: need to check if just the active flag changed and skip invalidate in that case
-        if (pano->getSrcImage(*it) != m_remapCache.getSrcDescription()) {
-            m_remapCache.invalidate(*it);
-        }
-    }
-    */
-
     m_imgsDirty = true;
-    /*
-    if (m_autoPreview) {
-        DEBUG_DEBUG("updating preview after image change");
-        updatePreview();
-    }
-    */
 }
 
-/*
-void PreviewPanel::SetDisplayedImages(const UIntSet & imgs)
-{
-    m_displayedImages = imgs;
-    if (m_autoPreview) {
-        updatePreview();
-    }
-}
-*/
-
 void PreviewPanel::SetBlendMode(BlendMode b)
 {
     m_blendMode = b;
@@ -260,11 +219,6 @@ void PreviewPanel::updatePreview()
     m_state_rendering = true;
     m_rerender = false;
 
-    long nthreads = wxConfigBase::Get()->Read(wxT("/Nona/NumberOfThreads"), wxThread::GetCPUCount());
-    if (nthreads < 1) nthreads = 1;
-    vigra_ext::ThreadManager::get().setNThreads(nthreads);
-
-
 	{
 	  // Even though the frame is hidden, the panel is not
 	  // so check the parent instead
@@ -278,19 +232,11 @@ void PreviewPanel::updatePreview()
 		}
 	  }
 	}
-//    bool seaming = wxConfigBase::Get()->Read("/PreviewPanel/UseSeaming",0l) != 0;
-
-    // temporary bitmap for our remapped image
-    // calculate the image size from panel widht, height from vfov
-
-//    long cor = wxConfigBase::Get()->Read("/PreviewPanel/correctDistortion",0l);
-//    bool corrLens = cor != 0;
-
     wxBusyCursor wait;
     double finalWidth = pano->getOptions().getWidth();
     double finalHeight = pano->getOptions().getHeight();
 
-    m_panoImgSize = Diff2D(GetClientSize().GetWidth(), GetClientSize().GetHeight());
+    m_panoImgSize = vigra::Diff2D(GetClientSize().GetWidth(), GetClientSize().GetHeight());
 
     double ratioPano = finalWidth / finalHeight;
     double ratioPanel = (double)m_panoImgSize.x / (double)m_panoImgSize.y;
@@ -307,50 +253,36 @@ void PreviewPanel::updatePreview()
         DEBUG_DEBUG("landscape: " << m_panoImgSize);
     }
 
-    PanoramaOptions opts = pano->getOptions();
+    HuginBase::PanoramaOptions opts = pano->getOptions();
     //don't use GPU for preview
     opts.remapUsingGPU = false;
     opts.setWidth(m_panoImgSize.x, false);
     opts.setHeight(m_panoImgSize.y);
-    //m_panoImgSize.y = opts.getHeight();
     // always use bilinear for preview.
 
     // reset ROI. The preview needs to draw the parts outside the ROI, too!
-    opts.setROI(Rect2D(opts.getSize()));
+    opts.setROI(vigra::Rect2D(opts.getSize()));
     opts.interpolator = vigra_ext::INTERP_BILINEAR;
 
     // create images
     wxImage panoImage(m_panoImgSize.x, m_panoImgSize.y);
     try {
-        vigra::BasicImageView<RGBValue<unsigned char> > panoImg8((RGBValue<unsigned char> *)panoImage.GetData(), panoImage.GetWidth(), panoImage.GetHeight());
-        FRGBImage panoImg(m_panoImgSize);
-        BImage alpha(m_panoImgSize);
-        // the empty panorama roi
-//        Rect2D panoROI;
+        vigra::BasicImageView<vigra::RGBValue<unsigned char> > panoImg8((vigra::RGBValue<unsigned char> *)panoImage.GetData(), panoImage.GetWidth(), panoImage.GetHeight());
+        vigra::FRGBImage panoImg(m_panoImgSize);
+        vigra::BImage alpha(m_panoImgSize);
+
         DEBUG_DEBUG("about to stitch images, pano size: " << m_panoImgSize);
-        UIntSet displayedImages = pano->getActiveImages();
+        HuginBase::UIntSet displayedImages = pano->getActiveImages();
         if (displayedImages.size() > 0) {
-            if (opts.outputMode == PanoramaOptions::OUTPUT_HDR) {
+            if (opts.outputMode == HuginBase::PanoramaOptions::OUTPUT_HDR) {
                 DEBUG_DEBUG("HDR output merge");
 
-                ReduceToHDRFunctor<RGBValue<float> > hdrmerge;
-                ReduceStitcher<FRGBImage, BImage> stitcher(*pano, *parentWindow);
+                vigra_ext::ReduceToHDRFunctor<vigra::RGBValue<float> > hdrmerge;
+                HuginBase::Nona::ReduceStitcher<vigra::FRGBImage, vigra::BImage> stitcher(*pano, parentWindow);
                 stitcher.stitch(opts, displayedImages,
                                 destImageRange(panoImg), destImage(alpha),
                                 m_remapCache,
                                 hdrmerge);
-                /*
-                std::vector<RemappedPanoImage<FRGBImage, BImage> *> remapped;
-                // get all remapped images
-                for (UIntSet::const_iterator it = displayedImages.begin();
-                     it != displayedImages.end(); ++it)
-                {
-                    remapped.push_back(m_remapCache.getRemapped(pano, opts, *it, *parentWindow));
-                }
-                reduceROIImages(remapped,
-                                destImageRange(panoImg), destImage(alpha),
-                                hdrmerge);
-                */
 #ifdef DEBUG_REMAP
 {
     vigra::ImageExportInfo exi( DEBUG_FILE_PREFIX "hugin04_preview_HDR_Reduce.tif"); \
@@ -364,73 +296,41 @@ void PreviewPanel::updatePreview()
 
                 // find min and max
                 vigra::FindMinMax<float> minmax;   // init functor
-                vigra::inspectImageIf(srcImageRange(panoImg), srcImage(alpha),
+                vigra::inspectImageIf(vigra::srcImageRange(panoImg), vigra::srcImage(alpha),
                                     minmax);
                 double min = std::max(minmax.min, 1e-6f);
                 double max = minmax.max;
 
-#if 0
-                for (int i=0; i<3; i++) {
-                    if (minmax.min[i]> 1e-6 && minmax.min[i] < min)
-                        min = minmax.min[i];
-                }
-                double max = DBL_MIN;
-                for (int i=0; i<3; i++) {
-                    if (minmax.max[i]> 1e-6 && minmax.max[i] > max)
-                        max = minmax.max[i];
-                }
-#endif
-
                 int mapping = wxConfigBase::Get()->Read(wxT("/ImageCache/MappingFloat"), HUGIN_IMGCACHE_MAPPING_FLOAT);
-                applyMapping(srcImageRange(panoImg), destImage(panoImg8), min, max, mapping);
+                vigra_ext::applyMapping(vigra::srcImageRange(panoImg), vigra::destImage(panoImg8), min, max, mapping);
 
             } else {
                     // LDR output
-    //            FileRemapper<BRGBImage, BImage> m;
+                vigra::ImageImportInfo::ICCProfile iccProfile;
                 switch (m_blendMode) {
                 case BLEND_COPY:
                 {
-                    StackingBlender blender;
-    //                SimpleStitcher<BRGBImage, BImage> stitcher(pano, *(MainFrame::Get()));
-                    SimpleStitcher<FRGBImage, BImage> stitcher(*pano, *parentWindow);
+                    HuginBase::Nona::StackingBlender blender;
+                    HuginBase::Nona::SimpleStitcher<vigra::FRGBImage, vigra::BImage> stitcher(*pano, parentWindow);
                     stitcher.stitch(opts, displayedImages,
                                     destImageRange(panoImg), destImage(alpha),
                                     m_remapCache,
                                     blender);
+                    iccProfile = stitcher.iccProfile;
                     break;
                 }
                 case BLEND_DIFFERENCE:
                 {
-                    ReduceToDifferenceFunctor<RGBValue<float> > func;
-                    ReduceStitcher<FRGBImage, BImage> stitcher(*pano, *parentWindow);
+                    HuginBase::Nona::ReduceToDifferenceFunctor<vigra::RGBValue<float> > func;
+                    HuginBase::Nona::ReduceStitcher<vigra::FRGBImage, vigra::BImage> stitcher(*pano, parentWindow);
                     stitcher.stitch(opts, displayedImages,
                                     destImageRange(panoImg), destImage(alpha),
                                     m_remapCache,
                                     func);
-                    break;
-    /*
-    
-                    WeightedStitcher<BRGBImage, BImage> stitcher(pano, *(MainFrame::Get()));
-                    stitcher.stitch(opts, m_displayedImages,
-                                    destImageRange(panoImg), destImage(alpha),
-                                    m_remapCache);
-                    break;
-    */
-                }
-    /*
-                case BLEND_DIFFERENCE:
-                {
-                    DifferenceBlender blender;
-                    SimpleStitcher<BRGBImage, BImage> stitcher(pano, *(MainFrame::Get()));
-                    stitcher.stitch(opts, m_displayedImages,
-                                    destImageRange(panoImg), destImage(alpha),
-                                    m_remapCache,
-                                    blender);
+                    iccProfile = stitcher.iccProfile;
                     break;
                 }
-    */
                 }
-
                 
 #ifdef DEBUG_REMAP
 {
@@ -444,7 +344,7 @@ void PreviewPanel::updatePreview()
 #endif
 
                 // apply default exposure and convert to 8 bit
-                SrcPanoImage src = pano->getSrcImage(0);
+                HuginBase::SrcPanoImage src = pano->getSrcImage(0);
 
                 // apply the exposure
                 double scale = 1.0/pow(2.0,opts.outputExposureValue);
@@ -453,22 +353,21 @@ void PreviewPanel::updatePreview()
                                       vigra::functor::Arg1()*vigra::functor::Param(scale));
 
                 DEBUG_DEBUG("LDR output, with response: " << src.getResponseType());
-                if (src.getResponseType() == SrcPanoImage::RESPONSE_LINEAR) {
-                    vigra::copyImage(srcImageRange(panoImg), destImage(panoImg8));
-//                    vigra::transformImage(srcImageRange(panoImg), destImage(panoImg8),
-//                                          vigra::functor::Arg1()*vigra::functor::Param(255));
+                if (src.getResponseType() == HuginBase::SrcPanoImage::RESPONSE_LINEAR) {
+                    vigra::transformImage(srcImageRange(panoImg), destImage(panoImg8),
+                                          vigra::functor::Arg1()*vigra::functor::Param(255));
                 } else {
                 // create suitable lut for response
                     typedef  std::vector<double> LUT;
                     LUT lut;
                     switch(src.getResponseType())
                     {
-                        case SrcPanoImage::RESPONSE_EMOR:
-                            EMoR::createEMoRLUT(src.getEMoRParams(), lut);
+                        case HuginBase::SrcPanoImage::RESPONSE_EMOR:
+                            vigra_ext::EMoR::createEMoRLUT(src.getEMoRParams(), lut);
                             break;
-                        case SrcPanoImage::RESPONSE_GAMMA:
+                        case HuginBase::SrcPanoImage::RESPONSE_GAMMA:
                             lut.resize(256);
-                            createGammaLUT(1/src.getGamma(), lut);
+                            vigra_ext::createGammaLUT(1/src.getGamma(), lut);
                             break;
                         default:
                             vigra_fail("Unknown or unsupported response function type");
@@ -478,11 +377,16 @@ void PreviewPanel::updatePreview()
                     for (size_t i=0; i < lut.size(); i++) 
                         lut[i] = lut[i]*255;
                     typedef vigra::RGBValue<float> FRGB;
-                    LUTFunctor<FRGB, LUT> lutf(lut);
+                    vigra_ext::LUTFunctor<FRGB, LUT> lutf(lut);
 
                     vigra::transformImage(srcImageRange(panoImg), destImage(panoImg8),
                                           lutf);
                 }
+                // apply color profiles
+                if (!iccProfile.empty() || huginApp::Get()->HasMonitorProfile())
+                {
+                    HuginBase::Color::CorrectImage(panoImage, iccProfile, huginApp::Get()->GetMonitorProfile());
+                };
             }
         }
 
@@ -503,11 +407,11 @@ void PreviewPanel::updatePreview()
 
     // update the transform for pano -> erect coordinates
     if (m_pano2erect) delete m_pano2erect;
-    SrcPanoImage src;
-    src.setProjection(SrcPanoImage::EQUIRECTANGULAR);
+    HuginBase::SrcPanoImage src;
+    src.setProjection(HuginBase::SrcPanoImage::EQUIRECTANGULAR);
     src.setHFOV(360);
-    src.setSize(Size2D(360,180));
-    m_pano2erect = new PTools::Transform;
+    src.setSize(vigra::Size2D(360,180));
+    m_pano2erect = new HuginBase::PTools::Transform;
     m_pano2erect->createTransform(src, opts);
 
     if (m_panoBitmap) {
@@ -533,8 +437,6 @@ void PreviewPanel::DrawPreview(wxDC & dc)
     }
     DEBUG_TRACE("");
 
-//    bool drawOutlines = wxConfigBase::Get()->Read("/PreviewPanel/drawOutlines",1l) != 0;
-
     int offsetX = 0;
     int offsetY = 0;
 
@@ -546,8 +448,13 @@ void PreviewPanel::DrawPreview(wxDC & dc)
         offsetY = (sz.GetHeight() - m_panoImgSize.y) / 2;
     }
 
-    dc.SetPen(wxPen(GetBackgroundColour(),1,wxSOLID));
+#if wxCHECK_VERSION(3,0,0)
+    dc.SetPen(wxPen(GetBackgroundColour(), 1, wxPENSTYLE_SOLID));
+    dc.SetBrush(wxBrush(GetBackgroundColour(), wxBRUSHSTYLE_SOLID));
+#else
+    dc.SetPen(wxPen(GetBackgroundColour(), 1, wxSOLID));
     dc.SetBrush(wxBrush(GetBackgroundColour(),wxSOLID));
+#endif
     dc.DrawRectangle(0, 0, offsetX, sz.GetHeight());
     dc.DrawRectangle(offsetX, 0, sz.GetWidth(), offsetY);
     dc.DrawRectangle(offsetX, sz.GetHeight() - offsetY,
@@ -560,8 +467,13 @@ void PreviewPanel::DrawPreview(wxDC & dc)
     dc.SetClippingRegion(offsetX, offsetY,
                          m_panoImgSize.x, m_panoImgSize.y);
 
-    dc.SetPen(wxPen(wxT("BLACK"),1,wxSOLID));
+#if wxCHECK_VERSION(3,0,0)
+    dc.SetPen(wxPen(wxT("BLACK"), 1, wxPENSTYLE_SOLID));
+    dc.SetBrush(wxBrush(wxT("BLACK"), wxBRUSHSTYLE_SOLID));
+#else
+    dc.SetPen(wxPen(wxT("BLACK"), 1, wxSOLID));
     dc.SetBrush(wxBrush(wxT("BLACK"),wxSOLID));
+#endif
     dc.DrawRectangle(offsetX, offsetY, m_panoImgSize.x, m_panoImgSize.y);
 
 
@@ -575,13 +487,13 @@ void PreviewPanel::DrawPreview(wxDC & dc)
         dc.DrawBitmap(*m_panoBitmap, offsetX, offsetY);
 
         // draw ROI
-        Size2D panoSize =  pano->getOptions().getSize();
-        Rect2D panoROI =  pano->getOptions().getROI();
+        vigra::Size2D panoSize = pano->getOptions().getSize();
+        vigra::Rect2D panoROI = pano->getOptions().getROI();
         if (panoROI != vigra::Rect2D(panoSize)) {
 
-            double scale = min(w/(double)panoSize.x, h/(double)panoSize.y);
-            Rect2D previewROI = Rect2D(panoROI.upperLeft()* scale, panoROI.lowerRight() * scale);
-            Rect2D screenROI = previewROI;
+            double scale = std::min(w/(double)panoSize.x, h/(double)panoSize.y);
+            vigra::Rect2D previewROI = vigra::Rect2D(panoROI.upperLeft()* scale, panoROI.lowerRight() * scale);
+            vigra::Rect2D screenROI = previewROI;
             screenROI.moveBy(offsetX, offsetY);
 
 
@@ -636,7 +548,11 @@ void PreviewPanel::DrawPreview(wxDC & dc)
                          m_panoImgSize.x, m_panoImgSize.y);
 
             // draw boundaries
+#if wxCHECK_VERSION(3,0,0)
+            dc.SetPen(wxPen(wxT("WHITE"), 1, wxPENSTYLE_SOLID));
+#else
             dc.SetPen(wxPen(wxT("WHITE"), 1, wxSOLID));
+#endif
             dc.SetLogicalFunction(wxINVERT);
 
             DEBUG_DEBUG("ROI scale factor: " << scale << " screen ROI: " << screenROI);
@@ -654,24 +570,16 @@ void PreviewPanel::DrawPreview(wxDC & dc)
         }
     }
 
-#if 0
-    // currently disabled
-    if (drawOutlines) {
-        for (UIntSet::iterator it = m_displayedImages.begin();
-             it != m_displayedImages.end();
-             ++it)
-        {
-            dc.SetPen(wxPen(wxT("GREY"), 1, wxSOLID));
-            DrawOutline(m_remapped[*it]->getOutline(), dc, offsetX, offsetY);
-        }
-    }
-#endif
     dc.DestroyClippingRegion();
     dc.SetClippingRegion(offsetX, offsetY,
                     m_panoImgSize.x, m_panoImgSize.y);
 
     // draw center lines over display
+#if wxCHECK_VERSION(3,0,0)
+    dc.SetPen(wxPen(wxT("WHITE"), 1, wxPENSTYLE_SOLID));
+#else
     dc.SetPen(wxPen(wxT("WHITE"), 1, wxSOLID));
+#endif
     dc.SetLogicalFunction(wxINVERT);
     dc.DrawLine(offsetX + w/2, offsetY,
                 offsetX + w/2, offsetY + h);
@@ -718,8 +626,8 @@ void PreviewPanel::mousePressLMBEvent(wxMouseEvent & e)
     r = RAD_TO_DEG(r);
     DEBUG_DEBUG("rotation angles pitch*yaw: " << y << " " << p << " " << r);
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(*pano, y, p, r)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::RotatePanoCmd(*pano, y, p, r)
         );
     if (!m_autoPreview) {
         ForceUpdate();
@@ -744,8 +652,8 @@ void PreviewPanel::mousePressRMBEvent(wxMouseEvent & e)
 
     DEBUG_DEBUG("roll correction: " << roll);
 
-    GlobalCmdHist::getInstance().addCommand(
-            new PT::RotatePanoCmd(*pano, 0, 0, roll)
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(
+            new PanoCommand::RotatePanoCmd(*pano, 0, 0, roll)
         );
     if (!m_autoPreview) {
         ForceUpdate();
@@ -757,25 +665,6 @@ void PreviewPanel::OnMouse(wxMouseEvent & e)
 {
     double yaw, pitch;
     mouse2erect(e.m_x, e.m_y,  yaw, pitch);
-    
-    /*
-    wxSize sz = GetClientSize();
-    int offsetX = 0;
-    int offsetY = 0;
-    if (sz.GetWidth() > m_panoImgSize.x) {
-        offsetX = (sz.GetWidth() - m_panoImgSize.x) / 2;
-    }
-    if (sz.GetHeight() > m_panoImgSize.y) {
-        offsetY = (sz.GetHeight() - m_panoImgSize.y) / 2;
-    }
-    double x = e.m_x - offsetX - m_panoImgSize.x/2;
-    double y = e.m_y - offsetY - m_panoImgSize.y/2;
-
-    int w = pano->getOptions().getWidth();
-    double scale = w/(double)m_panoImgSize.x;
-    x *= scale;
-    y *= scale;
-    */
 
     parentWindow->SetStatusText(_("Left click to define new center point, right click to move point to horizon."),0);
     parentWindow->SetStatusText(wxString::Format(wxT("%.1f %.1f"), yaw, pitch), 1);
@@ -783,9 +672,8 @@ void PreviewPanel::OnMouse(wxMouseEvent & e)
 
 void PreviewPanel::mouse2erect(int xm, int ym, double &xd, double & yd)
 {
-    int offsetX=0, offsetY=0;
-
     if (m_pano2erect) {
+        int offsetX=0, offsetY=0;
         wxSize sz = GetClientSize();
         if (sz.GetWidth() > m_panoImgSize.x) {
             offsetX = (sz.GetWidth() - m_panoImgSize.x) / 2;
@@ -800,20 +688,20 @@ void PreviewPanel::mouse2erect(int xm, int ym, double &xd, double & yd)
      }
 }
 
-void PreviewPanel::DrawOutline(const vector<FDiff2D> & points, wxDC & dc, int offX, int offY)
+void PreviewPanel::DrawOutline(const std::vector<hugin_utils::FDiff2D> & points, wxDC & dc, int offX, int offY)
 {
-    for (vector<FDiff2D>::const_iterator pnt = points.begin();
+    for (std::vector<hugin_utils::FDiff2D>::const_iterator pnt = points.begin();
          pnt != points.end() ;
          ++pnt)
     {
-        Diff2D point = pnt->toDiff2D();
+        vigra::Diff2D point = pnt->toDiff2D();
         if (point.x < 0) point.x = 0;
         if (point.y < 0) point.y = 0;
         if (point.x >= m_panoImgSize.x)
             point.x = m_panoImgSize.y-1;
         if (point.y >= m_panoImgSize.y)
             point.y = m_panoImgSize.y -1;
-        dc.DrawPoint(roundi(offX + point.x), roundi(offY + point.y));
+        dc.DrawPoint(hugin_utils::roundi(offX + point.x), hugin_utils::roundi(offY + point.y));
     }
 }
 
