@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -30,78 +30,98 @@
 
 #ifdef __WXMAC__
 #include <wx/sysopt.h>
+#include <wx/dir.h>
 #endif
 
 #include "panoinc.h"
 
-
 #include "hugin/config_defaults.h"
 #include "hugin/huginApp.h"
-#include "hugin/AssistantPanel.h"
 #include "hugin/ImagesPanel.h"
-#include "hugin/CropPanel.h"
 #include "hugin/MaskEditorPanel.h"
 #include "hugin/CPEditorPanel.h"
 #include "hugin/OptimizePhotometricPanel.h"
 #include "hugin/PanoPanel.h"
-#include "hugin/LensPanel.h"
 #include "hugin/ImagesList.h"
+#include "hugin/CPListFrame.h"
 #include "hugin/PreviewPanel.h"
 #include "hugin/GLPreviewFrame.h"
 #include "base_wx/PTWXDlg.h"
-#include "hugin/CommandHistory.h"
-#include "hugin/wxPanoCommand.h"
+#include "base_wx/CommandHistory.h"
+#include "base_wx/wxcms.h"
+#include "base_wx/wxPanoCommand.h"
 #include "hugin/HtmlWindow.h"
+#include "hugin/treelistctrl.h"
+#include "hugin/ImagesTree.h"
 
 #include "base_wx/platform.h"
 #include "base_wx/huginConfig.h"
 #ifdef __WXMSW__
-#include "wx/dir.h"
-#include "wx/cshelp.h"
+#include <wx/dir.h>
+#include <wx/cshelp.h>
+#include <wx/stdpaths.h>
+#if wxCHECK_VERSION(3,1,0)
+#include <wx/taskbarbutton.h>
+#endif
 #endif
 
 #include <tiffio.h>
 
 #include "AboutDialog.h"
 
-using namespace hugin_utils;
+//for natural sorting
+#include "hugin_utils/alphanum.h"
+#include "lensdb/LensDB.h"
 
-// utility functions
-bool str2double(wxString s, double & d)
+bool checkVersion(wxString v1, wxString v2)
 {
-    if (!stringToDouble(std::string(s.mb_str(wxConvLocal)), d)) {
-        wxLogError(_("Value must be numeric."));
-        return false;
-    }
-    return true;
-}
+    return doj::alphanum_comp(std::string(v1.mb_str(wxConvLocal)),std::string(v2.mb_str(wxConvLocal))) < 0;
+};
 
-wxString getDefaultProjectName(const Panorama & pano)
+wxString Components2Str(const HuginGraph::ImageGraph::Components & comp)
 {
-    bool t = (wxConfigBase::Get()->Read(wxT("ProjectNamingConvention"), HUGIN_PROJECT_NAMING_CONVENTION) != 0);
-    if(t){
-        if (pano.getNrOfImages() > 0) {
-            wxFileName first_img(wxString(pano.getImage(0).getFilename().c_str(),HUGIN_CONV_FILENAME));
-            if(first_img.GetDirCount()>0)
-                return first_img.GetPathWithSep()+first_img.GetDirs().Last();
+    wxString ret;
+    for (unsigned i=0; i < comp.size(); i++) {
+        ret.Append(wxT("["));
+        HuginGraph::ImageGraph::Components::value_type::const_iterator it = comp[i].begin();
+        while (it != comp[i].end())
+        {
+            unsigned int imgNr = *it;
+            ret << imgNr;
+            it++;
+            if (it != comp[i].end() && *it == imgNr + 1)
+            {
+                ret.Append(wxT("-"));
+                while (it != comp[i].end() && *it == imgNr + 1)
+                {
+                    ++it;
+                    ++imgNr;
+                };
+                ret << imgNr;
+                if (it != comp[i].end())
+                {
+                    ret.Append(wxT(", "));
+                };
+            }
             else
-                return first_img.GetPathWithSep()+wxT("pano");
-        } else {
-            return wxT("pano");
-        }
-    }else{
-        if (pano.getNrOfImages() > 0) {
-            wxFileName first_img(wxString(pano.getImage(0).getFilename().c_str(),HUGIN_CONV_FILENAME));
-            first_img.ClearExt();
-            wxFileName last_img(wxString(pano.getImage(pano.getNrOfImages()-1).getFilename().c_str(),HUGIN_CONV_FILENAME));
-            return first_img.GetFullPath() + wxT("-") + last_img.GetName();
-        } else {
-            return wxT("pano");
-        }
+            {
+                if (it != comp[i].end())
+                {
+                    ret.Append(wxT(", "));
+                };
+            };
+        };
+
+        ret.Append(wxT("]"));
+        if (i + 1 != comp.size())
+        {
+            ret.Append(wxT(", "));
+        };
     }
+    return ret;
 }
 
-#if _WINDOWS && defined Hugin_shared
+#if defined _WIN32 && defined Hugin_shared
 DEFINE_LOCAL_EVENT_TYPE( EVT_IMAGE_READY )
 #else
 DEFINE_EVENT_TYPE( EVT_IMAGE_READY )
@@ -118,6 +138,10 @@ huginApp::huginApp()
 {
     DEBUG_TRACE("ctor");
     m_this=this;
+    m_monitorProfile = NULL;
+#if wxUSE_ON_FATAL_EXCEPTION
+    wxHandleFatalExceptions();
+#endif
 }
 
 huginApp::~huginApp()
@@ -131,6 +155,12 @@ huginApp::~huginApp()
 	// todo: remove all listeners from the panorama object
 
 //    delete frame;
+    HuginBase::LensDB::LensDB::Clean();
+    // delete monitor profile
+    if (m_monitorProfile)
+    {
+        cmsCloseProfile(m_monitorProfile);
+    };
     DEBUG_TRACE("dtor end");
 }
 
@@ -159,17 +189,15 @@ bool huginApp::OnInit()
     wxHelpControllerHelpProvider* provider = new wxHelpControllerHelpProvider;
     wxHelpProvider::Set(provider);
 
-    wxString huginExeDir = getExePath(argv[0]);
-
-    wxString huginRoot;
-    wxFileName::SplitPath( huginExeDir, &huginRoot, NULL, NULL );
-
-    m_xrcPrefix = huginRoot + wxT("/share/hugin/xrc/");
-	m_DataDir = huginRoot + wxT("/share/hugin/data/");
-    m_utilsBinDir = huginRoot + wxT("/bin/");
+    wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+    m_utilsBinDir = exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
+    exePath.RemoveLastDir();
+    const wxString huginRoot=exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR);
+    m_xrcPrefix = huginRoot + wxT("share\\hugin\\xrc\\");
+    m_DataDir = huginRoot + wxT("share\\hugin\\data\\");
 
     // locale setup
-    locale.AddCatalogLookupPathPrefix(huginRoot + wxT("/share/locale"));
+    locale.AddCatalogLookupPathPrefix(huginRoot + wxT("share\\locale"));
 
 #elif defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
     // initialize paths
@@ -183,6 +211,22 @@ bool huginApp::OnInit()
         m_DataDir = thePath + wxT("/");
     }
 
+#ifdef HUGIN_HSI
+    // Set PYTHONHOME for the hsi module
+    {
+        wxString pythonHome = MacGetPathToBundledFrameworksDirectory() + wxT("/Python27.framework/Versions/Current");
+        if(! wxDir::Exists(pythonHome)){
+            wxMessageBox(wxString::Format(_("Directory '%s' does not exists"), pythonHome.c_str()));
+        } else {
+            wxUnsetEnv(wxT("PYTHONPATH"));
+            if(! wxSetEnv(wxT("PYTHONHOME"), pythonHome)){
+                wxMessageBox(_("Could not set environment variable PYTHONHOME"));
+            } else {
+                DEBUG_TRACE("PYTHONHOME set to " << pythonHome);
+            }
+        }
+    }
+#endif
     {
         wxString thePath = MacGetPathToBundledResourceFile(CFSTR("locale"));
         if(thePath != wxT(""))
@@ -192,7 +236,7 @@ bool huginApp::OnInit()
             return false;
         }
     }
-
+	
 #else
     // add the locale directory specified during configure
     m_xrcPrefix = wxT(INSTALL_XRC_DIR);
@@ -242,14 +286,7 @@ bool huginApp::OnInit()
     #else
 
     // add custom XRC handlers
-    wxXmlResource::Get()->AddHandler(new AssistantPanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new ImagesPanelXmlHandler());
-    wxXmlResource::Get()->AddHandler(new LensPanelXmlHandler());
-    wxXmlResource::Get()->AddHandler(new ImagesListImageXmlHandler());
-    wxXmlResource::Get()->AddHandler(new ImagesListLensXmlHandler());
-    wxXmlResource::Get()->AddHandler(new ImagesListCropXmlHandler());
-    wxXmlResource::Get()->AddHandler(new CropPanelXmlHandler());
-    wxXmlResource::Get()->AddHandler(new CenterCanvasXmlHandler());
     wxXmlResource::Get()->AddHandler(new CPEditorPanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new CPImageCtrlXmlHandler());
     wxXmlResource::Get()->AddHandler(new CPImagesComboBoxXmlHandler());
@@ -261,9 +298,15 @@ bool huginApp::OnInit()
     wxXmlResource::Get()->AddHandler(new PanoPanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new PreviewPanelXmlHandler());
     wxXmlResource::Get()->AddHandler(new HtmlWindowXmlHandler());
+#if wxCHECK_VERSION(2,9,0)
+    wxXmlResource::Get()->AddHandler(new wxcode::wxTreeListCtrlXmlHandler());
+#else
+    wxXmlResource::Get()->AddHandler(new wxTreeListCtrlXmlHandler());
+#endif
+    wxXmlResource::Get()->AddHandler(new ImagesTreeCtrlXmlHandler());
+    wxXmlResource::Get()->AddHandler(new CPListCtrlXmlHandler());
 
     // load XRC files
-    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("crop_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("cp_list_frame.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("preview_frame.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("edit_script_dialog.xrc"));
@@ -276,12 +319,13 @@ bool huginApp::OnInit()
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("optimize_photo_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("cp_editor_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("images_panel.xrc"));
-    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("lens_panel.xrc"));
-    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("assistant_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("main_frame.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("optimize_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("pano_panel.xrc"));
     wxXmlResource::Get()->Load(m_xrcPrefix + wxT("mask_editor_panel.xrc"));
+    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("lensdb_dialogs.xrc"));
+    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("image_variable_dlg.xrc"));
+    wxXmlResource::Get()->Load(m_xrcPrefix + wxT("dlg_warning.xrc"));
 #endif
 
 #ifdef __WXMAC__
@@ -290,23 +334,47 @@ bool huginApp::OnInit()
     m_macInitDone=false;
     m_macOpenFileOnStart=false;
 #endif
+    // read monitor profile
+    HuginBase::Color::GetMonitorProfile(m_monitorProfileName, m_monitorProfile);
     // create main frame
     frame = new MainFrame(NULL, pano);
     SetTopWindow(frame);
 
-    // restore layout
-    frame->RestoreLayoutOnNextResize();
-
     // setup main frame size, after it has been created.
     RestoreFramePosition(frame, wxT("MainFrame"));
 #ifdef __WXMSW__
     provider->SetHelpController(&frame->GetHelpController());
     frame->GetHelpController().Initialize(m_xrcPrefix+wxT("data/hugin_help_en_EN.chm"));
     frame->SendSizeEvent();
+#if wxCHECK_VERSION(3,1,0)
+    wxTaskBarJumpList jumpList;
+    wxFileName exeFile(wxStandardPaths::Get().GetExecutablePath());
+    exeFile.SetName("PTBatcherGUI");
+    wxTaskBarJumpListItem *item1 = new wxTaskBarJumpListItem(
+        NULL, wxTASKBAR_JUMP_LIST_TASK, _("Open Batch Processor"), exeFile.GetFullPath(), wxEmptyString,
+        _("Opens PTBatcher, the batch processor for Hugin's project files"),
+        exeFile.GetFullPath(), 0);
+    jumpList.GetTasks().Append(item1);
+    exeFile.SetName("calibrate_lens_gui");
+    wxTaskBarJumpListItem *item2 = new wxTaskBarJumpListItem(
+        NULL, wxTASKBAR_JUMP_LIST_TASK, _("Open Lens calibrate tool"), exeFile.GetFullPath(), wxEmptyString,
+        _("Opens Calibrate_lens_gui, a simple GUI for lens calibration"),
+        exeFile.GetFullPath(), 0);
+    jumpList.GetTasks().Append(item2);
+#endif
 #endif
 
+    // we are closing Hugin, if the top level window is deleted
+    SetExitOnFrameDelete(true);
     // show the frame.
-    frame->Show(TRUE);
+    if(frame->GetGuiLevel()==GUI_SIMPLE)
+    {
+        SetTopWindow(frame->getGLPreview());
+    }
+    else
+    {
+        frame->Show(TRUE);
+    };
 
     wxString cwd = wxFileName::GetCwd();
 
@@ -348,10 +416,23 @@ bool huginApp::OnInit()
     DEBUG_DEBUG("using temp dir: " << m_workDir.mb_str(wxConvLocal));
 
     // set some suitable defaults
-    GlobalCmdHist::getInstance().addCommand(new wxNewProjectCmd(pano));
-    GlobalCmdHist::getInstance().clear();
+    PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::wxNewProjectCmd(pano));
+    PanoCommand::GlobalCmdHist::getInstance().clear();
 
-    if (argc > 1) {
+    // suppress tiff warnings
+    TIFFSetWarningHandler(0);
+
+    if (argc > 1)
+    {
+#ifdef __WXMSW__
+        //on Windows we need to update the fast preview first
+        //otherwise there is an infinite loop when starting with a project file
+        //and closed panorama editor aka mainframe
+        if(frame->GetGuiLevel()==GUI_SIMPLE)
+        {
+            frame->getGLPreview()->Update();
+        };
+#endif
         wxFileName file(argv[1]);
         // if the first file is a project file, open it
         if (file.GetExt().CmpNoCase(wxT("pto")) == 0 ||
@@ -360,15 +441,15 @@ bool huginApp::OnInit()
         {
             if(file.IsRelative())
                 file.MakeAbsolute(cwd);
-	    // Loading the project file with set actualPath to its
-	    // parent directory.  (actualPath is used as starting
-	    // directory by many subsequent file selection dialogs.)
+            // Loading the project file with set actualPath to its
+            // parent directory.  (actualPath is used as starting
+            // directory by many subsequent file selection dialogs.)
             frame->LoadProjectFile(file.GetFullPath());
         } else {
             std::vector<std::string> filesv;
+            bool actualPathSet = false;
             for (int i=1; i< argc; i++) 
             {
-                bool actualPathSet = false;
 #if defined __WXMSW__
                 //expand wildcards
                 wxFileName fileList(argv[i]);
@@ -417,9 +498,14 @@ bool huginApp::OnInit()
                 } while (dir.GetNext(&foundFile));
 #endif
             }
-            GlobalCmdHist::getInstance().addCommand(
-                    new PT::wxAddImagesCmd(pano,filesv)
-                                                );
+            if(filesv.size()>0)
+            {
+                std::vector<PanoCommand::PanoCommand*> cmds;
+                cmds.push_back(new PanoCommand::wxAddImagesCmd(pano,filesv));
+                cmds.push_back(new PanoCommand::DistributeImagesCmd(pano));
+                cmds.push_back(new PanoCommand::CenterPanoCmd(pano));
+                PanoCommand::GlobalCmdHist::getInstance().addCommand(new PanoCommand::CombinedPanoCommand(pano, cmds));
+            }
         }
     }
 #ifdef __WXMAC__
@@ -443,12 +529,6 @@ bool huginApp::OnInit()
 		}
 	}
 
-    // suppress tiff warnings
-    TIFFSetWarningHandler(0);
-
-    pano.changeFinished();
-    pano.clearDirty();
-
     DEBUG_TRACE("=========================== huginApp::OnInit() end ===================");
     return true;
 }
@@ -456,6 +536,10 @@ bool huginApp::OnInit()
 int huginApp::OnExit()
 {
     DEBUG_TRACE("");
+#ifdef __WXMSW__    
+    delete wxHelpProvider::Set(NULL);
+#endif
+
     return wxApp::OnExit();
 }
 
@@ -508,6 +592,13 @@ void huginApp::MacOpenFile(const wxString &fileName)
 }
 #endif
 
+#if wxUSE_ON_FATAL_EXCEPTION
+void huginApp::OnFatalException()
+{
+    GenerateReport(wxDebugReport::Context_Exception);
+};
+#endif
+
 huginApp * huginApp::m_this = 0;
 
 
@@ -523,7 +614,7 @@ void RestoreFramePosition(wxTopLevelWindow * frame, const wxString & basename)
     int dx,dy;
     wxDisplaySize(&dx,&dy);
 
-#if ( __WXGTK__ ) &&  wxCHECK_VERSION(2,6,0)
+#if ( __WXGTK__ )
 // restoring the splitter positions properly when maximising doesn't work.
 // Disabling maximise on wxWidgets >= 2.6.0 and gtk
         //size
@@ -574,7 +665,7 @@ void StoreFramePosition(wxTopLevelWindow * frame, const wxString & basename)
 
     wxConfigBase * config = wxConfigBase::Get();
 
-#if ( __WXGTK__ ) &&  wxCHECK_VERSION(2,6,0)
+#if ( __WXGTK__ )
 // restoring the splitter positions properly when maximising doesn't work.
 // Disabling maximise on wxWidgets >= 2.6.0 and gtk
     
