@@ -16,8 +16,8 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -26,20 +26,19 @@
 
 #include "config.h"
 #include <huginapp/ImageCache.h>
+#include <lcms2.h>
+#include <algorithms/optimizer/ImageGraph.h>
 
 #include "hugin/MainFrame.h"
 
 // utility functions
-bool str2double(wxString s, double & d);
+wxString Components2Str(const HuginGraph::ImageGraph::Components & comp);
 
 /// Store window size and position in configfile/registry
 void StoreFramePosition(wxTopLevelWindow * frame, const wxString & basename);
 /// Restore window size and position from configfile/registry
 void RestoreFramePosition(wxTopLevelWindow * frame, const wxString & basename);
 
-///
-wxString getDefaultProjectName(const Panorama & pano);
-
 /** Resources Definition
  *
  */
@@ -48,7 +47,7 @@ wxString getDefaultProjectName(const Panorama & pano);
   void InitXmlResource();
 #endif
 
-#if _WINDOWS && defined Hugin_shared
+#if defined _WIN32 && defined Hugin_shared
 DECLARE_LOCAL_EVENT_TYPE( EVT_IMAGE_READY, )
 #else
 DECLARE_EVENT_TYPE( EVT_IMAGE_READY, )
@@ -148,14 +147,36 @@ public:
         return m_utilsBinDir;
     }
 
+    /** returns the monitor profile, if no monitor profile was found the sRGB profile is used instead */
+    const cmsHPROFILE& GetMonitorProfile() const
+    {
+        return m_monitorProfile;
+    }
+    /** return true if we found a suitable monitor profile and could loading it */
+    bool HasMonitorProfile() const
+    {
+        return !m_monitorProfileName.IsEmpty();
+    }
+    /** return filename of monitor profile */
+    const wxString& GetMonitorProfileName() const
+    {
+        return m_monitorProfileName;
+    };
+
 #ifdef __WXMAC__
     //Defined in wxApp.h; This one lets project file to be opened from Finder and other applications.
     void MacOpenFile(const wxString &fileName);
 #endif
 
+#if wxUSE_ON_FATAL_EXCEPTION
+#if wxCHECK_VERSION(3,1,0)
+    virtual void OnFatalException() wxOVERRIDE;
+#else
+    virtual void OnFatalException();
+#endif
+#endif
+
 private:
-
-
     /** locale for internationalisation */
     wxLocale locale;
 
@@ -168,7 +189,7 @@ private:
     static huginApp * m_this;
 
     // the model
-    Panorama pano;
+    HuginBase::Panorama pano;
 
 	// folder for xrc (GUI resources)
     wxString m_xrcPrefix;
@@ -176,6 +197,9 @@ private:
 	wxString m_DataDir;
 	// folder for CLI tools
     wxString m_utilsBinDir;
+    // monitor profile
+    wxString m_monitorProfileName;
+    cmsHPROFILE m_monitorProfile;
 
 #ifdef __WXMAC__
     bool m_macInitDone;
