@@ -19,8 +19,8 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
@@ -29,6 +29,13 @@
 #include "PTBatcherGUI.h"
 #include "FindPanoDialog.h"
 #include "FailedProjectsDialog.h"
+#ifdef __WXMSW__
+#include <powrprof.h>
+#pragma comment(lib, "PowrProf.lib")
+#if wxCHECK_VERSION(3,1,0)
+#include <wx/taskbarbutton.h>
+#endif
+#endif
 
 /* file drag and drop handler method */
 bool BatchDropTarget::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
@@ -93,6 +100,7 @@ BEGIN_EVENT_TABLE(BatchFrame, wxFrame)
     EVT_MENU(XRCID("menu_open"),BatchFrame::OnButtonOpenBatch)
     EVT_MENU(XRCID("menu_save"),BatchFrame::OnButtonSaveBatch)
     EVT_MENU(XRCID("menu_clear"),BatchFrame::OnButtonClear)
+    EVT_MENU(XRCID("menu_tray"), BatchFrame::OnMinimizeTrayMenu)
     EVT_MENU(XRCID("menu_exit"),BatchFrame::OnUserExit)
     EVT_MENU(XRCID("menu_help"),BatchFrame::OnButtonHelp)
     EVT_BUTTON(XRCID("button_addcommand"),BatchFrame::OnButtonAddCommand)
@@ -103,12 +111,12 @@ BEGIN_EVENT_TABLE(BatchFrame, wxFrame)
     EVT_BUTTON(XRCID("button_edit"),BatchFrame::OnButtonOpenWithHugin)
     EVT_BUTTON(XRCID("button_move_up"),BatchFrame::OnButtonMoveUp)
     EVT_BUTTON(XRCID("button_move_down"),BatchFrame::OnButtonMoveDown)
-    EVT_CHECKBOX(XRCID("cb_parallel"), BatchFrame::OnCheckParallel)
     EVT_CHECKBOX(XRCID("cb_overwrite"), BatchFrame::OnCheckOverwrite)
-    EVT_CHECKBOX(XRCID("cb_shutdown"), BatchFrame::OnCheckShutdown)
+    EVT_CHOICE(XRCID("choice_end"), BatchFrame::OnChoiceEnd)
     EVT_CHECKBOX(XRCID("cb_verbose"), BatchFrame::OnCheckVerbose)
     EVT_CHECKBOX(XRCID("cb_autoremove"), BatchFrame::OnCheckAutoRemove)
     EVT_CHECKBOX(XRCID("cb_autostitch"), BatchFrame::OnCheckAutoStitch)
+    EVT_CHECKBOX(XRCID("cb_savelog"), BatchFrame::OnCheckSaveLog)
     EVT_END_PROCESS(-1, BatchFrame::OnProcessTerminate)
     EVT_CLOSE(BatchFrame::OnClose)
     EVT_MENU(wxEVT_COMMAND_RELOAD_BATCH, BatchFrame::OnReloadBatch)
@@ -116,6 +124,7 @@ BEGIN_EVENT_TABLE(BatchFrame, wxFrame)
     EVT_COMMAND(wxID_ANY, EVT_BATCH_FAILED, BatchFrame::OnBatchFailed)
     EVT_COMMAND(wxID_ANY, EVT_INFORMATION, BatchFrame::OnBatchInformation)
     EVT_COMMAND(wxID_ANY, EVT_UPDATE_PARENT, BatchFrame::OnRefillListBox)
+    EVT_COMMAND(wxID_ANY, EVT_QUEUE_PROGRESS, BatchFrame::OnProgress)
     EVT_ICONIZE(BatchFrame::OnMinimize)
 END_EVENT_TABLE()
 
@@ -140,52 +149,80 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     // create tool bar
     SetToolBar(wxXmlResource::Get()->LoadToolBar(this, wxT("batch_toolbar")));
 
-    CreateStatusBar(1);
+    int widths[2] = { -1, 150 };
+    CreateStatusBar(2);
+    SetStatusWidths(2, widths);
     SetStatusText(_("Not doing much..."));
 
     // set the minimize icon
 #ifdef __WXMSW__
-    wxIcon myIcon(m_xrcPrefix + wxT("data/ptbatcher.ico"),wxBITMAP_TYPE_ICO);
+    m_iconNormal = wxIcon(m_xrcPrefix + wxT("data/ptbatcher.ico"), wxBITMAP_TYPE_ICO);
+    m_iconRunning = wxIcon(m_xrcPrefix + wxT("data/ptbatcher_running.ico"), wxBITMAP_TYPE_ICO);
+    m_iconPaused = wxIcon(m_xrcPrefix + wxT("data/ptbatcher_pause.ico"), wxBITMAP_TYPE_ICO);
 #else
-    wxIcon myIcon(m_xrcPrefix + wxT("data/ptbatcher.png"),wxBITMAP_TYPE_PNG);
+    m_iconNormal = wxIcon(m_xrcPrefix + wxT("data/ptbatcher.png"), wxBITMAP_TYPE_PNG);
+    m_iconRunning = wxIcon(m_xrcPrefix + wxT("data/ptbatcher_running.png"), wxBITMAP_TYPE_PNG);
+    m_iconPaused = wxIcon(m_xrcPrefix + wxT("data/ptbatcher_pause.png"), wxBITMAP_TYPE_PNG);
 #endif
-    SetIcon(myIcon);
+    SetIcon(m_iconNormal);
 
-#if defined __WXMSW__
-    //show tray icon only on windows
-    //because not all window manager on *nix provide a usable tray area
-#if wxCHECK_VERSION(2,9,0)
-    if(wxTaskBarIcon::IsAvailable())
+    m_batch = new Batch(this);
+#if wxCHECK_VERSION(2,9,4)
+    if(wxGetKeyState(WXK_COMMAND))
+#else
+    if(wxGetKeyState(WXK_CONTROL))
+#endif
     {
-        m_tray=new BatchTaskBarIcon();
-        m_tray->SetIcon(myIcon,_("Hugin's Batch processor"));
+#ifdef __WXMAC__
+        wxString text(_("You have pressed the Command key."));
+#else
+        wxString text(_("You have pressed the Control key."));
+#endif
+        text.Append(wxT("\n"));
+        text.Append(_("Should the loading of the batch queue be skipped?"));
+        if(wxMessageBox(text, 
+#ifdef __WXMSW__
+            wxT("PTBatcherGUI"),
+#else
+            wxEmptyString,
+#endif
+            wxYES_NO | wxICON_EXCLAMATION, NULL)==wxNO)
+        {
+            m_batch->LoadTemp();
+        };
     }
     else
     {
-        m_tray=NULL;
+        m_batch->LoadTemp();
     };
-#else
-    m_tray=new BatchTaskBarIcon();
-    if(m_tray->IsOk())
-    {
-        m_tray->SetIcon(myIcon,_("Hugin's Batch processor"));
-    }
-    else
-    {
-        m_tray=NULL;
-    };
-#endif
-#else
-    m_tray=NULL;
-#endif
-    m_batch = new Batch(this,wxTheApp->argv[0],true);
-    m_batch->gui = true;
-    m_batch->LoadTemp();
     if(m_batch->GetLastFile().length()==0)
     {
         m_batch->SaveTemp();
     }
     projListBox = XRCCTRL(*this,"project_listbox",ProjectListBox);
+    // fill at end list box, check which options are available
+    m_endChoice = XRCCTRL(*this, "choice_end", wxChoice);
+    m_endChoice->Clear();
+    m_endChoice->Append(_("Do nothing"), (void*)Batch::DO_NOTHING);
+    m_endChoice->Append(_("Close PTBatcherGUI"), (void*)Batch::CLOSE_PTBATCHERGUI);
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+    // there is no wxShutdown for wxMac
+    m_endChoice->Append(_("Shutdown computer"), (void*)Batch::SHUTDOWN);
+#endif
+#ifdef __WXMSW__
+    SYSTEM_POWER_CAPABILITIES pwrCap;
+    if (GetPwrCapabilities(&pwrCap))
+    {
+        if (pwrCap.SystemS3)
+        {
+            m_endChoice->Append(_("Suspend computer"), (void*)Batch::SUSPEND);
+        };
+        if (pwrCap.SystemS4 && pwrCap.HiberFilePresent)
+        {
+            m_endChoice->Append(_("Hibernate computer"), (void*)Batch::HIBERNATE);
+        };
+    };
+#endif
 
     //wxThreadHelper::Create is deprecated in wxWidgets 2.9+, but its
     // replacement, CreateThread, does not exist in 2.8. Pick one
@@ -199,6 +236,23 @@ BatchFrame::BatchFrame(wxLocale* locale, wxString xrc)
     //TO-DO: include a batch or project progress gauge?
     projListBox->Fill(m_batch);
     SetDropTarget(new BatchDropTarget());
+
+    m_tray = NULL;
+#if wxCHECK_VERSION(3,0,0)
+    if (wxTaskBarIcon::IsAvailable())
+    {
+        // minimize to tray is by default disabled
+        // activate only if task bar icon is available
+        GetMenuBar()->Enable(XRCID("menu_tray"), true);
+        bool minTray;
+        // tray icon is disabled by default 
+        wxConfigBase::Get()->Read(wxT("/BatchFrame/minimizeTray"), &minTray, false);
+        GetMenuBar()->Check(XRCID("menu_tray"), minTray);
+        UpdateTrayIcon(minTray);
+    }
+#endif
+
+    UpdateTaskBarProgressBar();
 }
 
 void* BatchFrame::Entry()
@@ -211,8 +265,14 @@ void* BatchFrame::Entry()
         wxFileName aFile(m_batch->GetLastFile());
         if(!aFile.FileExists())
         {
+#if wxCHECK_VERSION(3,0,0)
+            // for thread safety wxWidgets recommends using wxQueueEvent
+            // but this function is not available for older wxWidgets versions
+            wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH));
+#else
             wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH);
-            wxPostEvent(this,evt);
+            wxPostEvent(this, evt);
+#endif
         }
         else
         {
@@ -220,18 +280,33 @@ void* BatchFrame::Entry()
             aFile.GetTimes(NULL,NULL,&create);
             if(create.IsLaterThan(m_batch->GetLastFileDate()))
             {
+#if wxCHECK_VERSION(3,0,0)
+                wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH));
+#else
                 wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_RELOAD_BATCH);
-                wxPostEvent(this,evt);
+                wxPostEvent(this, evt);
+#endif
             };
         };
         //update project list box
+#if wxCHECK_VERSION(3,0,0)
+        wxQueueEvent(this, new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_UPDATE_LISTBOX));
+#else
         wxCommandEvent evt(wxEVT_COMMAND_MENU_SELECTED, wxEVT_COMMAND_UPDATE_LISTBOX);
-        wxPostEvent(this,evt);
+        wxPostEvent(this, evt);
+#endif
         GetThread()->Sleep(1000);
     }
     return 0;
 }
 
+wxStatusBar* BatchFrame::OnCreateStatusBar(int number, long style, wxWindowID id, const wxString& name)
+{
+    m_progStatusBar = new ProgressStatusBar(this, id, style, name);
+    m_progStatusBar->SetFieldsCount(number);
+    return m_progStatusBar;
+}
+
 bool BatchFrame::IsRunning()
 {
     return m_batch->IsRunning();
@@ -351,15 +426,20 @@ void BatchFrame::OnButtonAddToStitchingQueue(wxCommandEvent& event)
     wxFileDialog dlg(0,
                      _("Specify project source file(s)"),
                      defaultdir, wxT(""),
-                     _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                     _("Project files (*.pto)|*.pto|All files (*)|*"),
                      wxFD_OPEN | wxFD_MULTIPLE, wxDefaultPosition);
-    dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/BatchFrame/actualPath"),wxT("")));
+    dlg.SetDirectory(defaultdir);
 
     if (dlg.ShowModal() == wxID_OK)
     {
-        wxConfig::Get()->Write(wxT("/BatchFrame/actualPath"), dlg.GetDirectory());  // remember for later
         wxArrayString paths;
         dlg.GetPaths(paths);
+#ifdef __WXGTK__
+        //workaround a bug in GTK, see https://bugzilla.redhat.com/show_bug.cgi?id=849692 and http://trac.wxwidgets.org/ticket/14525
+        wxConfig::Get()->Write(wxT("/BatchFrame/actualPath"), wxPathOnly(paths[0]));  // remember for later
+#else
+        wxConfig::Get()->Write(wxT("/BatchFrame/actualPath"), dlg.GetDirectory());  // remember for later
+#endif
         for(unsigned int i=0; i<paths.GetCount(); i++)
         {
             AddToList(paths.Item(i));
@@ -374,15 +454,21 @@ void BatchFrame::OnButtonAddToAssistantQueue(wxCommandEvent& event)
     wxFileDialog dlg(0,
                      _("Specify project source file(s)"),
                      defaultdir, wxT(""),
-                     _("Project files (*.pto,*.ptp,*.pts,*.oto)|*.pto;*.ptp;*.pts;*.oto;|All files (*)|*"),
+                     _("Project files (*.pto)|*.pto|All files (*)|*"),
                      wxFD_OPEN | wxFD_MULTIPLE, wxDefaultPosition);
-    dlg.SetDirectory(wxConfigBase::Get()->Read(wxT("/BatchFrame/actualPath"),wxT("")));
+    dlg.SetDirectory(defaultdir);
 
     if (dlg.ShowModal() == wxID_OK)
     {
-        wxConfig::Get()->Write(wxT("/BatchFrame/actualPath"), dlg.GetDirectory());  // remember for later
         wxArrayString paths;
         dlg.GetPaths(paths);
+#ifdef __WXGTK__
+        //workaround a bug in GTK, see https://bugzilla.redhat.com/show_bug.cgi?id=849692 and http://trac.wxwidgets.org/ticket/14525
+        wxConfig::Get()->Write(wxT("/BatchFrame/actualPath"), wxPathOnly(paths[0]));  // remember for later
+#else
+        wxConfig::Get()->Write(wxT("/BatchFrame/actualPath"), dlg.GetDirectory());  // remember for later
+#endif
+
         for(unsigned int i=0; i<paths.GetCount(); i++)
         {
             AddToList(paths.Item(i),Project::DETECTING);
@@ -401,16 +487,21 @@ void BatchFrame::AddDirToList(wxString aDir)
     for(unsigned int i=0; i<projects.GetCount(); i++)
     {
         m_batch->AddProjectToBatch(projects.Item(i));
-        projListBox->AppendProject(m_batch->GetProject(m_batch->GetProjectCount()-1));
+        Project* proj=m_batch->GetProject(m_batch->GetProjectCount()-1);
+        if(!proj->isAligned)
+        {
+            proj->target=Project::DETECTING;
+        };
+        projListBox->AppendProject(proj);
     };
     m_batch->SaveTemp();
-    SetStatusText(_("Added projects from dir ")+aDir);
+    SetStatusText(wxString::Format(_("Added projects from dir %s"), aDir.c_str()));
 };
 
 void BatchFrame::AddToList(wxString aFile,Project::Target target)
 {
     wxFileName name(aFile);
-    m_batch->AddProjectToBatch(aFile,name.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR) + name.GetName(),target);
+    m_batch->AddProjectToBatch(aFile,wxT(""),target);
     wxString s;
     switch(target)
     {
@@ -433,6 +524,10 @@ void BatchFrame::OnButtonCancel(wxCommandEvent& event)
     m_cancelled = true;
     m_batch->CancelBatch();
     SetStatusInformation(_("Batch stopped"),true);
+    if (m_tray)
+    {
+        m_tray->SetIcon(m_iconNormal, _("Hugin's Batch processor"));
+    };
 }
 
 void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
@@ -451,13 +546,31 @@ void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
             {
                 while(containsInvalidCharacters(dlg.GetPath()))
                 {
-                    wxMessageBox(wxString::Format(_("The given filename contains one of the following invalid characters: %s\nHugin can not work with this filename. Please enter a valid filename."),getInvalidCharacters().c_str()),
-                                 _("Error"),wxOK | wxICON_EXCLAMATION,this);
+                    wxArrayString list;
+                    list.Add(dlg.GetPath());
+                    ShowFilenameWarning(this, list);
                     if(dlg.ShowModal()!=wxID_OK)
                     {
                         return;
                     }
                 };
+                wxFileName prefix(dlg.GetPath());
+                while (!prefix.IsDirWritable())
+                {
+                    wxMessageBox(wxString::Format(_("You have no permissions to write in folder \"%s\".\nPlease select another folder for the final output."), prefix.GetPath().c_str()),
+#ifdef __WXMSW__
+                        wxT("PTBatcherGUI"),
+#else
+                        wxT(""),
+#endif
+                        wxOK | wxICON_INFORMATION);
+                    if (dlg.ShowModal() != wxID_OK)
+                    {
+                        return;
+                    };
+                    prefix = dlg.GetPath();
+                };
+
                 wxString outname(dlg.GetPath());
                 ChangePrefix(selIndex,outname);
                 //SetStatusText(_T("Changed prefix for "+projListBox->GetSelectedProject()));
@@ -466,7 +579,7 @@ void BatchFrame::OnButtonChangePrefix(wxCommandEvent& event)
         }
         else
         {
-            SetStatusText(_("The prefix of an assistant target can not be changed."));
+            SetStatusText(_("The prefix of an assistant target cannot be changed."));
             wxBell();
         };
     }
@@ -514,7 +627,7 @@ void BatchFrame::OnButtonHelp(wxCommandEvent& event)
 {
     DEBUG_TRACE("");
 #ifdef __WXMSW__
-    GetHelpController().DisplaySection(wxT("Hugin_Batch_Processor.html"));
+    GetHelpController().DisplayHelpPage(wxT("Hugin_Batch_Processor.html"));
 #else
     if (m_help == 0)
     {
@@ -600,17 +713,13 @@ void BatchFrame::OnButtonOpenBatch(wxCommandEvent& event)
 
 void BatchFrame::OnButtonOpenWithHugin(wxCommandEvent& event)
 {
-#ifdef __WINDOWS__
-    wxString huginPath = getExePath(wxGetApp().argv[0])+wxFileName::GetPathSeparator();
-#else
-    wxString huginPath = _T("");	//we call hugin directly without path on linux
-#endif
+    const wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
     if(projListBox->GetSelectedIndex()!=-1)
         if(projListBox->GetText(projListBox->GetSelectedIndex(),0).Cmp(_T(""))!=0)
 #ifdef __WXMAC__
             wxExecute(_T("open -b net.sourceforge.hugin.hugin \"" + projListBox->GetSelectedProject()+_T("\"")));
 #else
-            wxExecute(huginPath+_T("hugin \"" + projListBox->GetSelectedProject()+_T("\" -notips")));
+            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+_T("hugin \"" + projListBox->GetSelectedProject()+_T("\" -notips")));
 #endif
         else
         {
@@ -620,18 +729,18 @@ void BatchFrame::OnButtonOpenWithHugin(wxCommandEvent& event)
     {
         //ask user if he/she wants to load an empty project
         wxMessageDialog message(this,_("No project selected. Open Hugin without project?"),
-#ifdef _WINDOWS
+#ifdef _WIN32
                                 _("PTBatcherGUI"),
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxNO | wxICON_INFORMATION );
+                                wxYES_NO | wxICON_INFORMATION );
         if(message.ShowModal() == wxID_YES)
         {
 #ifdef __WXMAC__
             wxExecute(_T("open -b net.sourceforge.hugin.hugin"));
 #else
-            wxExecute(huginPath+_T("hugin"));
+            wxExecute(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR)+_T("hugin"));
 #endif
         }
     }
@@ -646,18 +755,27 @@ void BatchFrame::OnButtonPause(wxCommandEvent& event)
             m_batch->PauseBatch();
             GetToolBar()->ToggleTool(XRCID("tool_pause"),true);
             SetStatusInformation(_("Batch paused"),true);
+            if (m_tray)
+            {
+                m_tray->SetIcon(m_iconPaused, _("Pausing processing Hugin's batch queue"));
+            };
         }
         else
         {
             m_batch->PauseBatch();
             GetToolBar()->ToggleTool(XRCID("tool_pause"),false);
             SetStatusInformation(_("Continuing batch..."),true);
+            if (m_tray)
+            {
+                m_tray->SetIcon(m_iconRunning, _("Processing Hugin's batch queue"));
+            };
         }
     }
     else //if no projects are running, we deactivate the button
     {
         GetToolBar()->ToggleTool(XRCID("tool_pause"),false);
     }
+    UpdateTaskBarProgressBar();
 }
 
 void BatchFrame::OnButtonRemoveComplete(wxCommandEvent& event)
@@ -665,13 +783,13 @@ void BatchFrame::OnButtonRemoveComplete(wxCommandEvent& event)
     bool removeErrors=false;
     if(!m_batch->NoErrors())
     {
-        wxMessageDialog message(this,_("There are failed projects in the list.\nRemove them also?"),
-#ifdef _WINDOWS
+        wxMessageDialog message(this,_("There are failed projects in the list.\nRemove them too?"),
+#ifdef _WIN32
                                 _("PTBatcherGUI"),
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxNO | wxICON_INFORMATION );
+                                wxYES_NO | wxICON_INFORMATION );
         if(message.ShowModal()==wxID_YES)
         {
             removeErrors=true;
@@ -698,12 +816,12 @@ void BatchFrame::OnButtonRemoveFromList(wxCommandEvent& event)
         if(m_batch->GetStatus(selIndex)==Project::RUNNING || m_batch->GetStatus(selIndex)==Project::PAUSED)
         {
             wxMessageDialog message(this, _("Cannot remove project in progress.\nDo you want to cancel it?"),
-#ifdef _WINDOWS
+#ifdef _WIN32
                                     _("PTBatcherGUI"),
 #else
                                     wxT(""),
 #endif
-                                    wxYES | wxCANCEL | wxICON_INFORMATION);
+                                    wxYES_NO | wxICON_INFORMATION);
             if(message.ShowModal()==wxID_YES)
             {
                 OnButtonSkip(event);
@@ -711,7 +829,7 @@ void BatchFrame::OnButtonRemoveFromList(wxCommandEvent& event)
         }
         else
         {
-            SetStatusText(_("Removed project ")+projListBox->GetSelectedProject());
+            SetStatusText(wxString::Format(_("Removed project %s"), projListBox->GetSelectedProject().c_str()));
             projListBox->Deselect(selIndex);
             projListBox->DeleteItem(selIndex);
             m_batch->RemoveProjectAtIndex(selIndex);
@@ -733,12 +851,12 @@ void BatchFrame::OnButtonReset(wxCommandEvent& event)
         if(m_batch->GetStatus(selIndex)==Project::RUNNING || m_batch->GetStatus(selIndex)==Project::PAUSED)
         {
             wxMessageDialog message(this, _("Cannot reset project in progress.\nDo you want to cancel it?"),
-#ifdef _WINDOWS
+#ifdef _WIN32
                                     _("PTBatcherGUI"),
 #else
                                     wxT(""),
 #endif
-                                    wxYES | wxCANCEL | wxICON_INFORMATION);
+                                    wxYES_NO | wxICON_INFORMATION);
             if(message.ShowModal()==wxID_YES)
             {
                 OnButtonSkip(event);
@@ -747,7 +865,7 @@ void BatchFrame::OnButtonReset(wxCommandEvent& event)
         else
         {
             m_batch->SetStatus(selIndex,Project::WAITING);
-            SetStatusText(_("Reset project ")+projListBox->GetSelectedProject());
+            SetStatusText(wxString::Format(_("Reset project %s"), projListBox->GetSelectedProject().c_str()));
         }
     }
     else
@@ -762,12 +880,12 @@ void BatchFrame::OnButtonResetAll(wxCommandEvent& event)
     if(m_batch->GetRunningCount()!=0)
     {
         wxMessageDialog message(this, _("Cannot reset projects in progress.\nDo you want to cancel the batch?"),
-#ifdef _WINDOWS
+#ifdef _WIN32
                                 _("PTBatcherGUI"),
 #else
                                 wxT(""),
 #endif
-                                wxYES | wxCANCEL | wxICON_INFORMATION);
+                                wxYES_NO | wxICON_INFORMATION);
         if(message.ShowModal()==wxID_YES)
         {
             OnButtonCancel(event);
@@ -803,7 +921,7 @@ void BatchFrame::OnButtonSaveBatch(wxCommandEvent& event)
                      _("Specify batch file to save"),
                      defaultdir, wxT(""),
                      _("Batch file (*.ptb)|*.ptb;|All files (*)|*"),
-                     wxFD_SAVE, wxDefaultPosition);
+                     wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition);
     if (dlg.ShowModal() == wxID_OK)
     {
         wxConfig::Get()->Write(wxT("/BatchFrame/batchPath"), dlg.GetDirectory());  // remember for later
@@ -854,22 +972,91 @@ void BatchFrame::OnButtonSkip(wxCommandEvent& event)
     }
 }
 
+void SelectEndTask(wxControlWithItems* list, const Batch::EndTask endTask)
+{
+    for (unsigned int i = 0; i<list->GetCount(); i++)
+    {
+        if (static_cast<Batch::EndTask>((size_t)list->GetClientData(i)) == endTask)
+        {
+            list->SetSelection(i);
+            return;
+        };
+    };
+    list->SetSelection(0);
+};
+
 void BatchFrame::SetCheckboxes()
 {
     wxConfigBase* config=wxConfigBase::Get();
     int i;
-    i=config->Read(wxT("/BatchFrame/ParallelCheck"), 0l);
-    XRCCTRL(*this,"cb_parallel",wxCheckBox)->SetValue(i!=0);
+    // read older version
+#if defined __WXMAC__ || defined __WXOSX_COCOA__
+    i = 0;
+#else
     i=config->Read(wxT("/BatchFrame/ShutdownCheck"), 0l);
-    XRCCTRL(*this,"cb_shutdown",wxCheckBox)->SetValue(i!=0);
+#endif
+    if (i != 0)
+    {
+        SelectEndTask(m_endChoice, Batch::SHUTDOWN);
+        m_batch->atEnd = Batch::SHUTDOWN;
+    };
+    config->DeleteEntry(wxT("/BatchFrame/ShutdownCheck"));
+    // now read current version
+    i = config->Read(wxT("/BatchFrame/AtEnd"), 0l);
+#if defined __WXMAC__ || defined __WXOSX_COCOA__
+    // wxWidgets for MacOS does not support wxShutdown
+    if (i == Batch::SHUTDOWN)
+    {
+        i = 0;
+    };
+#endif
+    m_batch->atEnd = static_cast<Batch::EndTask>(i);
+    SelectEndTask(m_endChoice, m_batch->atEnd);
     i=config->Read(wxT("/BatchFrame/OverwriteCheck"), 0l);
     XRCCTRL(*this,"cb_overwrite",wxCheckBox)->SetValue(i!=0);
+    m_batch->overwrite=(i!=0);
     i=config->Read(wxT("/BatchFrame/VerboseCheck"), 0l);
     XRCCTRL(*this,"cb_verbose",wxCheckBox)->SetValue(i!=0);
-    i=config->Read(wxT("/BatchFrame/AutoRemoveCheck"), 0l);
+    m_batch->verbose=(i!=0);
+    i=config->Read(wxT("/BatchFrame/AutoRemoveCheck"), 1l);
     XRCCTRL(*this,"cb_autoremove",wxCheckBox)->SetValue(i!=0);
+    m_batch->autoremove=(i!=0);
     i=config->Read(wxT("/BatchFrame/AutoStitchCheck"), 0l);
     XRCCTRL(*this,"cb_autostitch",wxCheckBox)->SetValue(i!=0);
+    m_batch->autostitch=(i!=0);
+    i=config->Read(wxT("/BatchFrame/SaveLog"), 0l);
+    XRCCTRL(*this, "cb_savelog",wxCheckBox)->SetValue(i!=0);
+    m_batch->saveLog=(i!=0);
+};
+
+Batch::EndTask BatchFrame::GetEndTask()
+{
+    return static_cast<Batch::EndTask>((size_t)m_endChoice->GetClientData(m_endChoice->GetSelection()));
+};
+
+bool BatchFrame::GetCheckOverwrite()
+{
+    return XRCCTRL(*this,"cb_overwrite",wxCheckBox)->IsChecked();
+};
+
+bool BatchFrame::GetCheckVerbose()
+{
+    return XRCCTRL(*this,"cb_verbose",wxCheckBox)->IsChecked();
+};
+
+bool BatchFrame::GetCheckAutoRemove()
+{
+    return XRCCTRL(*this,"cb_autoremove",wxCheckBox)->IsChecked();
+};
+
+bool BatchFrame::GetCheckAutoStitch()
+{
+    return XRCCTRL(*this,"cb_autostitch",wxCheckBox)->IsChecked();
+};
+
+bool BatchFrame::GetCheckSaveLog()
+{
+    return XRCCTRL(*this,"cb_savelog",wxCheckBox)->IsChecked();
 };
 
 void BatchFrame::OnCheckOverwrite(wxCommandEvent& event)
@@ -886,32 +1073,10 @@ void BatchFrame::OnCheckOverwrite(wxCommandEvent& event)
     }
 }
 
-void BatchFrame::OnCheckParallel(wxCommandEvent& event)
+void BatchFrame::OnChoiceEnd(wxCommandEvent& event)
 {
-    if(event.IsChecked())
-    {
-        m_batch->parallel = true;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ParallelCheck"), 1l);
-    }
-    else
-    {
-        m_batch->parallel = false;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ParallelCheck"), 0l);
-    }
-}
-
-void BatchFrame::OnCheckShutdown(wxCommandEvent& event)
-{
-    if(event.IsChecked())
-    {
-        m_batch->shutdown = true;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ShutdownCheck"), 1l);
-    }
-    else
-    {
-        m_batch->shutdown = false;
-        wxConfigBase::Get()->Write(wxT("/BatchFrame/ShutdownCheck"), 0l);
-    }
+    m_batch->atEnd = static_cast<Batch::EndTask>((size_t)m_endChoice->GetClientData(event.GetSelection()));
+    wxConfigBase::Get()->Write(wxT("/BatchFrame/AtEnd"), static_cast<long>(m_batch->atEnd));
 }
 
 void BatchFrame::OnCheckVerbose(wxCommandEvent& event)
@@ -949,7 +1114,6 @@ void BatchFrame::OnCheckAutoRemove(wxCommandEvent& event)
     config->Flush();
 };
 
-
 void BatchFrame::OnCheckAutoStitch(wxCommandEvent& event)
 {
     m_batch->autostitch=event.IsChecked();
@@ -965,8 +1129,42 @@ void BatchFrame::OnCheckAutoStitch(wxCommandEvent& event)
     config->Flush();
 };
 
+void BatchFrame::OnCheckSaveLog(wxCommandEvent& event)
+{
+    m_batch->saveLog=event.IsChecked();
+    wxConfigBase* config=wxConfigBase::Get();
+    if(m_batch->saveLog)
+    {
+        config->Write(wxT("/BatchFrame/SaveLog"), 1l);
+    }
+    else
+    {
+        config->Write(wxT("/BatchFrame/SaveLog"), 0l);
+    }
+    config->Flush();
+};
+
 void BatchFrame::OnClose(wxCloseEvent& event)
 {
+    // check size of batch queue
+    if (m_batch->GetProjectCount() > 500)
+    {
+        wxMessageDialog message(this, _("The batch queue contains many items.\nThis can have negative effects on performance.\nShould the batch queue be cleared now?"),
+#ifdef __WXMSW__
+            _("PTBatcherGUI"),
+#else
+            wxT(""),
+#endif
+            wxYES_NO | wxICON_INFORMATION);
+#if wxCHECK_VERSION(3,0,0)
+        message.SetYesNoLabels(_("Clear batch queue now"), _("Keep batch queue"));
+#endif
+        if (message.ShowModal() == wxID_YES)
+        {
+            m_batch->ClearBatch();
+            m_batch->SaveTemp();
+        };
+    };
     //save windows position
     wxConfigBase* config=wxConfigBase::Get();
     if(IsMaximized())
@@ -990,21 +1188,24 @@ void BatchFrame::OnClose(wxCloseEvent& event)
     }
     config->Flush();
     m_closeThread = true;
-    this->GetThread()->Wait();
+    if (this->GetThread() && this->GetThread()->IsRunning())
+    {
+        this->GetThread()->Wait();
+    };
 #ifndef __WXMSW__
     delete m_help;
 #endif
     if(m_tray!=NULL)
     {
         delete m_tray;
+        m_tray = NULL;
     }
     this->Destroy();
 }
 
 void BatchFrame::PropagateDefaults()
 {
-    m_batch->parallel=GetCheckParallel();
-    m_batch->shutdown=GetCheckShutdown();
+    m_batch->atEnd = GetEndTask();
     m_batch->overwrite=GetCheckOverwrite();
     m_batch->verbose=GetCheckVerbose();
     m_batch->autoremove=GetCheckAutoRemove();
@@ -1016,6 +1217,10 @@ void BatchFrame::RunBatch()
     if(!IsRunning())
     {
         SetStatusInformation(_("Starting batch"),true);
+        if (m_tray)
+        {
+            m_tray->SetIcon(m_iconRunning, _("Processing Hugin's batch queue"));
+        };
     }
     m_batch->RunBatch();
 }
@@ -1049,6 +1254,10 @@ void BatchFrame::OnProcessTerminate(wxProcessEvent& event)
     if(m_batch->GetRunningCount()==1)
     {
         GetToolBar()->ToggleTool(XRCID("tool_pause"),false);
+        if (m_tray)
+        {
+            m_tray->SetIcon(m_iconNormal, _("Hugin's Batch processor"));
+        };
     }
     event.Skip();
 }
@@ -1079,13 +1288,25 @@ void BatchFrame::RestoreSize()
 
 void BatchFrame::OnBatchFailed(wxCommandEvent& event)
 {
-    FailedProjectsDialog failedProjects_dlg(this,m_batch,m_xrcPrefix);
-    failedProjects_dlg.ShowModal();
+    if(m_batch->GetFailedProjectsCount()>0)
+    {
+        if (m_tray)
+        {
+            m_tray->SetIcon(m_iconNormal, _("Hugin's Batch processor"));
+        };
+        FailedProjectsDialog failedProjects_dlg(this, m_batch, m_xrcPrefix);
+        failedProjects_dlg.ShowModal();
+    };
 };
 
 void BatchFrame::OnBatchInformation(wxCommandEvent& e)
 {
     SetStatusInformation(e.GetString(),true);
+    if (m_tray && e.GetInt() == 1)
+    {
+        // batch finished, reset icon in task bar
+        m_tray->SetIcon(m_iconNormal, _("Hugin's Batch processor"));
+    };
 };
 
 void BatchFrame::SetStatusInformation(wxString status,bool showBalloon)
@@ -1109,6 +1330,36 @@ void BatchFrame::SetStatusInformation(wxString status,bool showBalloon)
     };
 };
 
+void BatchFrame::OnProgress(wxCommandEvent& e)
+{
+    m_progStatusBar->SetProgress(e.GetInt());
+    UpdateTaskBarProgressBar();
+};
+
+void BatchFrame::UpdateTaskBarProgressBar()
+{
+#if defined __WXMSW__ && wxCHECK_VERSION(3,1,0)
+    // provide also a feedback in task bar if available
+    if (IsShown())
+    {
+        wxTaskBarButton* taskBarButton = MSWGetTaskBarButton();
+        if (taskBarButton != NULL)
+        {
+            if (m_progStatusBar->GetProgress() < 0)
+            {
+                taskBarButton->SetProgressValue(wxTASKBAR_BUTTON_NO_PROGRESS);
+            }
+            else
+            {
+                taskBarButton->SetProgressRange(100);
+                taskBarButton->SetProgressState(m_batch->IsPaused() ? wxTASKBAR_BUTTON_PAUSED : wxTASKBAR_BUTTON_NORMAL);
+                taskBarButton->SetProgressValue(m_progStatusBar->GetProgress());
+            };
+        };
+    };
+#endif
+};
+
 void BatchFrame::OnMinimize(wxIconizeEvent& e)
 {
     //hide/show window in taskbar when minimizing
@@ -1129,11 +1380,21 @@ void BatchFrame::OnMinimize(wxIconizeEvent& e)
         else
         {
             m_batch->verbose=XRCCTRL(*this,"cb_verbose",wxCheckBox)->IsChecked();
+            UpdateTaskBarProgressBar();
         };
         m_batch->ShowOutput(m_batch->verbose);
     }
     else //don't hide window if no tray icon
     {
+#if wxCHECK_VERSION(2,9,0)
+        if (!e.IsIconized())
+#else
+        if (!e.Iconized())
+#endif
+        {
+            // window is restored, update progress indicators
+            UpdateTaskBarProgressBar();
+        };
         e.Skip();
     };
 };
@@ -1163,3 +1424,47 @@ void BatchFrame::OnRefillListBox(wxCommandEvent& event)
         };
     };
 };
+
+void BatchFrame::OnMinimizeTrayMenu(wxCommandEvent& event)
+{
+    UpdateTrayIcon(event.IsChecked());
+    wxConfigBase::Get()->Write(wxT("/BatchFrame/minimizeTray"), event.IsChecked());
+}
+
+void BatchFrame::UpdateTrayIcon(const bool createTrayIcon)
+{
+    if (createTrayIcon)
+    {
+        // create tray icon only if it not exists
+        if (m_tray)
+        {
+            delete m_tray;
+            m_tray = NULL;
+        }
+        m_tray = new BatchTaskBarIcon();
+        if (m_batch->IsRunning())
+        {
+            m_tray->SetIcon(m_iconRunning, _("Processing Hugin's batch queue"));
+        }
+        else
+        {
+            if (m_batch->IsPaused())
+            {
+                m_tray->SetIcon(m_iconPaused, _("Pausing processing Hugin's batch queue"));
+            }
+            else
+            {
+                m_tray->SetIcon(m_iconNormal, _("Hugin's Batch processor"));
+            }
+        }
+    }
+    else
+    {
+        // destroy tray icon
+        if (m_tray)
+        {
+            delete m_tray;
+            m_tray = NULL;
+        }
+    }
+}
