@@ -19,15 +19,17 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "PTBatcherGUI.h"
 #ifdef __WXMSW__
-#include "wx/cshelp.h"
+#include <wx/cshelp.h>
+#include <wx/stdpaths.h>
 #endif
+#include "lensdb/LensDB.h"
 
 // make wxwindows use this class as the main application
 IMPLEMENT_APP(PTBatcherGUI)
@@ -39,6 +41,9 @@ END_EVENT_TABLE()
 
 bool PTBatcherGUI::OnInit()
 {
+#if wxUSE_ON_FATAL_EXCEPTION
+    wxHandleFatalExceptions();
+#endif
     // Required to access the preferences of hugin
     SetAppName(wxT("hugin"));
 
@@ -55,15 +60,13 @@ bool PTBatcherGUI::OnInit()
 
     // setup the environment for the different operating systems
 #if defined __WXMSW__
-    wxString huginExeDir = getExePath(argv[0]);
-
-    wxString huginRoot;
-    wxFileName::SplitPath(huginExeDir, &huginRoot, NULL, NULL);
-    m_xrcPrefix = wxString(huginRoot + wxT("/share/hugin/xrc/"));
+    wxFileName exePath(wxStandardPaths::Get().GetExecutablePath());
+    exePath.RemoveLastDir();
+    const wxString huginRoot(exePath.GetPath(wxPATH_GET_VOLUME | wxPATH_GET_SEPARATOR));
+    m_xrcPrefix = huginRoot + wxT("share\\hugin\\xrc\\");
 
     // locale setup
-    m_locale.AddCatalogLookupPathPrefix(huginRoot + wxT("/share/locale"));
-
+    m_locale.AddCatalogLookupPathPrefix(huginRoot + wxT("share\\locale"));
 #elif defined __WXMAC__ && defined MAC_SELF_CONTAINED_BUNDLE
     {
         wxString exec_path = MacGetPathToBundledResourceFile(CFSTR("xrc"));
@@ -74,6 +77,7 @@ bool PTBatcherGUI::OnInit()
         else
         {
             wxMessageBox(_("xrc directory not found in bundle"), _("Fatal Error"));
+            return false;
         }
 
         wxString thePath = MacGetPathToBundledResourceFile(CFSTR("locale"));
@@ -117,6 +121,8 @@ bool PTBatcherGUI::OnInit()
         wxXmlResource::Get()->Load(m_xrcPrefix + wxT("batch_frame.xrc"));
         wxXmlResource::Get()->Load(m_xrcPrefix + wxT("batch_toolbar.xrc"));
         wxXmlResource::Get()->Load(m_xrcPrefix + wxT("batch_menu.xrc"));
+        wxXmlResource::Get()->Load(m_xrcPrefix + wxT("lensdb_dialogs.xrc"));
+        wxXmlResource::Get()->Load(m_xrcPrefix + wxT("dlg_warning.xrc"));
     };
 
     // parse arguments
@@ -130,13 +136,12 @@ bool PTBatcherGUI::OnInit()
             wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP
         },
         { wxCMD_LINE_SWITCH, "b", "batch",  "run batch immediately" },
-        { wxCMD_LINE_SWITCH, "p", "parallel",  "run batch projects in parallel" },
         { wxCMD_LINE_SWITCH, "o", "overwrite",  "overwrite previous files without asking" },
         { wxCMD_LINE_SWITCH, "s", "shutdown",  "shutdown computer after batch is complete" },
         { wxCMD_LINE_SWITCH, "v", "verbose",  "show verbose output when processing projects" },
         { wxCMD_LINE_SWITCH, "a", "assistant", "run the assistant on the given projects" },
         {
-            wxCMD_LINE_PARAM,  NULL, NULL, _("stitch_project.pto [output prefix]|assistant_project.pto"),
+            wxCMD_LINE_PARAM,  NULL, NULL, "stitch_project.pto [output prefix]|assistant_project.pto",
             wxCMD_LINE_VAL_STRING, wxCMD_LINE_PARAM_OPTIONAL + wxCMD_LINE_PARAM_MULTIPLE
         },
         { wxCMD_LINE_NONE }
@@ -146,7 +151,6 @@ bool PTBatcherGUI::OnInit()
             wxCMD_LINE_VAL_NONE, wxCMD_LINE_OPTION_HELP
         },
         { wxCMD_LINE_SWITCH, wxT("b"), wxT("batch"),  wxT("run batch immediately") },
-        { wxCMD_LINE_SWITCH, wxT("p"), wxT("parallel"),  wxT("run batch projects in parallel") },
         { wxCMD_LINE_SWITCH, wxT("o"), wxT("overwrite"),  wxT("overwrite previous files without asking") },
         { wxCMD_LINE_SWITCH, wxT("s"), wxT("shutdown"),  wxT("shutdown computer after batch is complete") },
         { wxCMD_LINE_SWITCH, wxT("v"), wxT("verbose"),  wxT("show verbose output when processing projects") },
@@ -176,7 +180,7 @@ bool PTBatcherGUI::OnInit()
     wxClient client;
     wxConnectionBase* conn;
     wxString servername;
-#ifdef __WINDOWS__
+#ifdef _WIN32
     servername=name;
 #else
     servername=wxFileName::GetTempDir()+wxFileName::GetPathSeparator()+name+wxT(".ipc");
@@ -194,6 +198,10 @@ bool PTBatcherGUI::OnInit()
         {
             m_frame->Show(true);
         }
+        else
+        {
+            m_frame->SetStatusInformation(_("PTBatcherGUI started"), true);
+        };
         m_server = new BatchIPCServer();
         if (!m_server->Create(servername))
         {
@@ -314,7 +322,8 @@ bool PTBatcherGUI::OnInit()
         }
     }
 
-#ifdef __WXMAC__
+#if 0 //ifdef __WXMAC__
+    // see PTBatcherGUI::MacOpenFile for explanation
     m_macFileNameToOpenOnStart = wxT("");
     wxYield();
     if(m_macFileNameToOpenOnStart != wxT(""))
@@ -327,13 +336,13 @@ bool PTBatcherGUI::OnInit()
     if(IsFirstInstance)
     {
         wxConfigBase* config=wxConfigBase::Get();
-        if (parser.Found(wxT("p")))
-        {
-            config->Write(wxT("/BatchFrame/ParallelCheck"), 1l);
-        }
         if (parser.Found(wxT("s")))
         {
-            config->Write(wxT("/BatchFrame/ShutdownCheck"), 1l);
+            config->DeleteEntry(wxT("/BatchFrame/ShutdownCheck"));
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+            // wxMac has not wxShutdown
+            config->Write(wxT("/BatchFrame/AtEnd"), static_cast<long>(Batch::SHUTDOWN));
+#endif
         }
         if (parser.Found(wxT("o")))
         {
@@ -347,13 +356,12 @@ bool PTBatcherGUI::OnInit()
     }
     else
     {
-        if (parser.Found(wxT("p")))
-        {
-            conn->Request(wxT("SetParallelCheck"));
-        }
         if (parser.Found(wxT("s")))
         {
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+            // wxMac has not wxShutdown
             conn->Request(wxT("SetShutdownCheck"));
+#endif
         }
         if (parser.Found(wxT("o")))
         {
@@ -389,11 +397,19 @@ bool PTBatcherGUI::OnInit()
 
 int PTBatcherGUI::OnExit()
 {
+    HuginBase::LensDB::LensDB::Clean();
     delete m_checker;
     delete m_server;
     return 0;
 }
 
+#if wxUSE_ON_FATAL_EXCEPTION
+void PTBatcherGUI::OnFatalException()
+{
+    GenerateReport(wxDebugReport::Context_Exception);
+};
+#endif
+
 void PTBatcherGUI::OnItemActivated(wxListEvent& event)
 {
     wxCommandEvent dummy;
@@ -421,8 +437,13 @@ void PTBatcherGUI::OnKeyDown(wxKeyEvent& event)
 
 }
 
-#ifdef __WXMAC__
+#if 0 //ifdef __WXMAC__
 // wx calls this method when the app gets "Open file" AppleEvent
+// this does not work with wxWidgets 2.9.3, Cocoa variant. The project is added twice.
+// From the doc it is recommended to react only on MacOpenFile
+// So we should  to skip to code concerning command line parsing in OnInit, and parse it here
+// But this does not work. MacOpenFile is not called when PTBatcherGUI is already running
+// So for the moment I leave the code in OnInit also for Mac
 void PTBatcherGUI::MacOpenFile(const wxString& fileName)
 {
     if(!m_frame)
@@ -440,12 +461,14 @@ void PTBatcherGUI::MacOpenFile(const wxString& fileName)
 #if wxCHECK_VERSION(2,9,0)
 #define RETURNEMPTYSTRING return wxEmptyString
 const void* BatchIPCConnection::OnRequest(const wxString& topic, const wxString& item, size_t* size, wxIPCFormat format)
-#else
-#define RETURNEMPTYSTRING return wxT("")
-wxChar* BatchIPCConnection::OnRequest(const wxString& topic, const wxString& item, int* size, wxIPCFormat format)
-#endif
 {
     *size=wxNO_LEN;
+#else
+#define RETURNEMPTYSTRING return ((wxChar*) "")
+wxChar* BatchIPCConnection::OnRequest(const wxString& topic, const wxString& item, int* size, wxIPCFormat format)
+{
+    *size=-1;
+#endif
     BatchFrame* MyBatchFrame=wxGetApp().GetFrame();
     if(item.Left(1)==wxT("A"))
     {
@@ -464,18 +487,17 @@ wxChar* BatchIPCConnection::OnRequest(const wxString& topic, const wxString& ite
     };
     wxCommandEvent event;
     event.SetInt(1);
-    if(item==wxT("SetParallelCheck"))
-        if(!MyBatchFrame->GetCheckParallel())
-        {
-            MyBatchFrame->OnCheckParallel(event);
-            MyBatchFrame->SetCheckboxes();
-        };
+#if !defined __WXMAC__ && !defined __WXOSX_COCOA__
+    // wxMac has not wxShutdown
     if(item==wxT("SetShutdownCheck"))
-        if(!MyBatchFrame->GetCheckShutdown())
+        if (MyBatchFrame->GetEndTask()!=Batch::SHUTDOWN)
         {
-            MyBatchFrame->OnCheckShutdown(event);
+            wxCommandEvent choiceEvent;
+            choiceEvent.SetInt(Batch::SHUTDOWN);
+            MyBatchFrame->OnChoiceEnd(choiceEvent);
             MyBatchFrame->SetCheckboxes();
         };
+#endif
     if(item==wxT("SetOverwriteCheck"))
         if(!MyBatchFrame->GetCheckOverwrite())
         {
