@@ -19,17 +19,30 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "ProjectListBox.h"
 
+enum
+{
+    ID_CHANGE_PREFIX=wxID_HIGHEST+200,
+    ID_RESET_PROJECT=wxID_HIGHEST+201,
+    ID_EDIT_PROJECT=wxID_HIGHEST+202,
+    ID_REMOVE_PROJECT=wxID_HIGHEST+203
+};
+
 BEGIN_EVENT_TABLE(ProjectListBox, wxListCtrl)
     EVT_LIST_ITEM_SELECTED(wxID_ANY, ProjectListBox::OnSelect)
     EVT_LIST_ITEM_DESELECTED(wxID_ANY, ProjectListBox::OnDeselect)
     EVT_LIST_COL_END_DRAG(wxID_ANY, ProjectListBox::OnColumnWidthChange)
+    EVT_CONTEXT_MENU(ProjectListBox::OnContextMenu)
+    EVT_MENU(ID_CHANGE_PREFIX, ProjectListBox::OnChangePrefix)
+    EVT_MENU(ID_RESET_PROJECT, ProjectListBox::OnResetProject)
+    EVT_MENU(ID_EDIT_PROJECT, ProjectListBox::OnEditProject)
+    EVT_MENU(ID_REMOVE_PROJECT, ProjectListBox::OnRemoveProject)
 END_EVENT_TABLE()
 
 bool ProjectListBox::Create(wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style, const wxString& name)
@@ -113,9 +126,11 @@ void ProjectListBox::AppendProject(Project* project)
 
 void ProjectListBox::ChangePrefix(int index, wxString newPrefix)
 {
-    if(columns.Index(PREFIX)!=wxNOT_FOUND)
+    int colIndex=columns.Index(PREFIX);
+    if(colIndex!=wxNOT_FOUND)
     {
-        this->SetItem(index,columns.Index(PREFIX),newPrefix);
+        Project* project=m_batch->GetProject(index);
+        this->SetItem(index,colIndex,this->GetAttributeString(colIndex,project));
     }
 }
 
@@ -271,32 +286,42 @@ wxString ProjectListBox::GetAttributeString(int i, Project* project)
     switch(i)
     {
         case 0:
-            return wxString::Format(_T("%d"),project->id);
+            return wxString::Format(_T("%ld"),project->id);
         case 1:
             return project->path;
         case 2:
             if(project->target==Project::STITCHING)
             {
-                return project->prefix;
+                //make prefix relative to project path
+                wxFileName prefix(project->prefix);
+                wxFileName projectFile(project->path);
+                prefix.MakeRelativeTo(projectFile.GetPath());
+                return prefix.GetFullPath();
             }
             else
             {
                 return _("Assistant");
             };
         case 7:
-            return project->GetStatusText();	//all following cases default to an empty string if file is missing
+            return project->GetStatusText();	
+        //all following cases default to an empty string if file is missing
         case 3:
             if(project->status!=Project::MISSING)
             {
-                return project->modDate.FormatDate()+_T(", ")+project->modDate.FormatTime();
-            }
+                if (project->modDate.IsValid())
+                {
+                    return project->modDate.Format();
+                };
+            };
+            return wxEmptyString;
         case 4:
             if(project->status!=Project::MISSING)
             {
                 str = GetLongerFormatName(project->options.outputImageType);
                 str = str+wxT(" (.")+wxString::FromAscii(project->options.outputImageType.c_str())+wxT(")");
                 return str;
-            }
+            };
+            return wxEmptyString;
         case 5:
             if(project->status!=Project::MISSING)
             {
@@ -311,6 +336,7 @@ wxString ProjectListBox::GetAttributeString(int i, Project* project)
                     return _T("");
                 }
             };
+            return wxEmptyString;
         case 6:
             if(project->status!=Project::MISSING)
             {
@@ -319,6 +345,7 @@ wxString ProjectListBox::GetAttributeString(int i, Project* project)
                 str = str << project->options.getHeight();
                 return str;
             }
+            return wxEmptyString;
         default:
             return _T("");
     }
@@ -364,8 +391,60 @@ void ProjectListBox::OnSelect(wxListEvent& event)
     m_selected = ((wxListEvent)event).GetIndex();
 }
 
+// functions for context menu
+void ProjectListBox::OnContextMenu(wxContextMenuEvent& e)
+{
+    if(m_selected!=-1)
+    {
+        wxPoint point = e.GetPosition();
+        // if from keyboard
+        if((point.x==-1) && (point.y==-1))
+        {
+            wxSize size = GetSize();
+            point.x = size.x / 2;
+            point.y = size.y / 2;
+        }
+        else
+        {
+            point = ScreenToClient(point);
+        }
+        wxMenu menu;
+        menu.Append(ID_CHANGE_PREFIX, _("Change prefix"));
+        menu.Append(ID_RESET_PROJECT, _("Reset project"));
+        menu.Append(ID_EDIT_PROJECT, _("Edit with Hugin"));
+        menu.Append(ID_REMOVE_PROJECT, _("Remove"));
+        PopupMenu(&menu, point.x, point.y);
+    };
+};
+
+void ProjectListBox::OnChangePrefix(wxCommandEvent& e)
+{
+    wxCommandEvent ev(wxEVT_COMMAND_BUTTON_CLICKED, XRCID("button_prefix"));
+    GetParent()->GetEventHandler()->AddPendingEvent(ev);
+};
+
+void ProjectListBox::OnResetProject(wxCommandEvent& e)
+{
+    wxCommandEvent ev(wxEVT_COMMAND_BUTTON_CLICKED, XRCID("button_reset"));
+    GetParent()->GetEventHandler()->AddPendingEvent(ev);
+};
+
+void ProjectListBox::OnEditProject(wxCommandEvent& e)
+{
+    wxCommandEvent ev(wxEVT_COMMAND_BUTTON_CLICKED, XRCID("button_edit"));
+    GetParent()->GetEventHandler()->AddPendingEvent(ev);
+};
+
+void ProjectListBox::OnRemoveProject(wxCommandEvent& e)
+{
+    wxCommandEvent ev(wxEVT_COMMAND_TOOL_CLICKED, XRCID("tool_remove"));
+    GetParent()->GetEventHandler()->AddPendingEvent(ev);
+};
+
 const wxString ProjectListBox::fileFormat[] = {_T("JPEG"),
+        _T("JPEG_m"),
         _T("PNG"),
+        _T("PNG_m"),
         _T("TIFF"),
         _T("TIFF_m"),
         _T("TIFF_mask"),
