@@ -19,15 +19,20 @@
  *  General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include "ImageCache.h"
 
 #include <iostream>
+#include "hugin_config.h"
+#ifdef USE_CXX11_THREAD
+#include <thread>
+#else
 #include <boost/thread/thread.hpp>
+#endif
 #include <vigra/inspectimage.hxx>
 #include <vigra/accessor.hxx>
 #include <vigra/functorexpression.hxx>
@@ -41,10 +46,6 @@
 
 namespace HuginBase {
     
-using namespace std;
-using namespace vigra::functor;
-
-
 template <class T1>
 class GetRange
 {
@@ -128,12 +129,12 @@ ImageCache * ImageCache::instance = NULL;
 
 void ImageCache::removeImage(const std::string & filename)
 {
-    map<string, EntryPtr>::iterator it = images.find(filename);
+    std::map<std::string, EntryPtr>::iterator it = images.find(filename);
     if (it != images.end()) {
         images.erase(it);
     }
 
-    string sfilename = filename + string("_small");
+    std::string sfilename = filename + std::string("_small");
     it = images.find(sfilename);
     if (it != images.end()) {
         images.erase(it);
@@ -144,7 +145,7 @@ void ImageCache::removeImage(const std::string & filename)
     do {
         // found. xyz
         PyramidKey key(filename,level);
-        map<string, vigra::BImage*>::iterator it = pyrImages.find(key.toString());
+        std::map<std::string, vigra::BImage*>::iterator it = pyrImages.find(key.toString());
         found = (it != pyrImages.end());
         if (found) {
             delete it->second;
@@ -154,12 +155,18 @@ void ImageCache::removeImage(const std::string & filename)
     } while (found);
 }
 
+std::string ImageCache::PyramidKey::toString()
+{
+    std::ostringstream s;
+    s << filename << level;
+    return s.str();
+};
 
 void ImageCache::flush()
 {
     images.clear();
 
-    for (map<string, vigra::BImage*>::iterator it = pyrImages.begin();
+    for (std::map<std::string, vigra::BImage*>::iterator it = pyrImages.begin();
          it != pyrImages.end();
          ++it)
     {
@@ -178,40 +185,40 @@ void ImageCache::softFlush()
     long imgMem = 0;
 
     std::map<std::string, EntryPtr>::iterator imgIt;
-    for(imgIt=images.begin(); imgIt != images.end(); imgIt++) {
+    for(imgIt=images.begin(); imgIt != images.end(); ++imgIt) {
 #ifdef DEBUG
-        cout << "Image: " << imgIt->first << std::endl;
-        cout << "CacheEntry: " << imgIt->second.use_count() << "last access: " << imgIt->second->lastAccess;
+        std::cout << "Image: " << imgIt->first << std::endl;
+        std::cout << "CacheEntry: " << imgIt->second.use_count() << "last access: " << imgIt->second->lastAccess;
 #endif
         if (imgIt->second->image8) {
             imgMem += imgIt->second->image8->width() * imgIt->second->image8->height() * 3;
 #ifdef DEBUG
-            cout << " 8bit: " << imgIt->second->image8.use_count();
+            std::cout << " 8bit: " << imgIt->second->image8.use_count();
 #endif
         }
         if (imgIt->second->image16) {
             imgMem += imgIt->second->image16->width() * imgIt->second->image16->height() * 3*2;
 #ifdef DEBUG
-            cout << " 16bit: " << imgIt->second->image8.use_count();
+            std::cout << " 16bit: " << imgIt->second->image8.use_count();
 #endif
         }
         if (imgIt->second->imageFloat) {
             imgMem += imgIt->second->imageFloat->width() * imgIt->second->imageFloat->height() * 3 * 4;
 #ifdef DEBUG
-            cout << " float: " << imgIt->second->imageFloat.use_count() ;
+            std::cout << " float: " << imgIt->second->imageFloat.use_count() ;
 #endif
         }
         if (imgIt->second->mask) {
             imgMem += imgIt->second->mask->width() * imgIt->second->mask->height();
 #ifdef DEBUG
-            cout << " mask: " << imgIt->second->mask.use_count() << std:: endl;
+            std::cout << " mask: " << imgIt->second->mask.use_count() << std::endl;
 #endif
         }
     }
 
     long pyrMem = 0;
     std::map<std::string, vigra::BImage*>::iterator pyrIt;
-    for(pyrIt=pyrImages.begin(); pyrIt != pyrImages.end(); pyrIt++) {
+    for(pyrIt=pyrImages.begin(); pyrIt != pyrImages.end(); ++pyrIt) {
         pyrMem += pyrIt->second->width() * pyrIt->second->height();
     }
 
@@ -229,7 +236,7 @@ void ImageCache::softFlush()
         // use least recently uses strategy.
         // sort images by their access time
         std::map<int,std::string> accessMap;
-        for (map<string, EntryPtr>::iterator it = images.begin();
+        for (std::map<std::string, EntryPtr>::iterator it = images.begin();
              it != images.end();
              ++it)
         {
@@ -245,15 +252,15 @@ void ImageCache::softFlush()
         }
         while (purgeAmount > purgedMem) {
             bool deleted = false;
-            if (pyrImages.size() > 0) {
+            if (!pyrImages.empty()) {
                 vigra::BImage * imgPtr = (*(pyrImages.begin())).second;
                 purgedMem += imgPtr->width() * imgPtr->height();
                 delete imgPtr;
                 pyrImages.erase(pyrImages.begin());
                 deleted = true;
-            } else if (accessMap.size() > 0) {
+            } else if (!accessMap.empty()) {
                 std::map<int,std::string>::iterator accIt = accessMap.begin();
-                map<string, EntryPtr>::iterator it = images.find(accIt->second);
+                std::map<std::string, EntryPtr>::iterator it = images.find(accIt->second);
                 // check for uniqueness.
                 if (it != images.end()) {
                     DEBUG_DEBUG("soft flush: removing image: " << it->first);
@@ -280,10 +287,6 @@ void ImageCache::softFlush()
                 break;
             }
         }
-//        m_progress->progressMessage(
-//            wxString::Format(_("Purged %d MB from image cache. Current cache usage: %d MB"),
-//                             purgedMem>>20, (usedMem - purgedMem)>>20
-//                ).c_str(),0);
         DEBUG_DEBUG("purged: " << (purgedMem>>20) << " MB, memory used for images: " << ((usedMem - purgedMem)>>20) << " MB");
 
     }
@@ -393,9 +396,9 @@ void ImageCache::importAndConvertImage(const vigra::ImageImportInfo & info,
         double scale = 1.0/vigra_ext::LUTTraits<SrcPixelType>::max();
 //        DestPixelType factor(scale);
         vigra_ext::Multiply<double> f(scale);
-        transformImage(dest.first, dest.first+ vigra::Diff2D(info.width(), info.height()), dest.second,
-                       dest.first, dest.second,
-                       Arg1()*Param(scale));
+        vigra::transformImage(dest.first, dest.first + vigra::Diff2D(info.width(), info.height()), dest.second,
+            dest.first, dest.second,
+            vigra::functor::Arg1()*vigra::functor::Param(scale));
     }
 }
 
@@ -487,10 +490,6 @@ void ImageCache::importAndConvertAlphaImage(const vigra::ImageImportInfo & info,
                                             vigra::pair<MaskIterator, MaskAccessor> mask,
                                             const std::string & type)
 {
-    typedef typename DestAccessor::value_type DestPixelType;
-    typedef typename DestPixelType::value_type DestComponentType;
-    typedef SrcPixelType SrcComponentType;
-
     if (type == "FLOAT" || type == "DOUBLE" ) {
         // import image as it is
         vigra::importImageAlpha(info, dest, mask);
@@ -498,10 +497,9 @@ void ImageCache::importAndConvertAlphaImage(const vigra::ImageImportInfo & info,
         // integer image.. scale to 0 .. 1
         vigra::importImageAlpha(info, dest, mask);
         double scale = 1.0/vigra_ext::LUTTraits<SrcPixelType>::max();
-        DestPixelType factor(scale);
-        transformImage(dest.first, dest.first+ vigra::Diff2D(info.width(), info.height()),  dest.second, 
-                       dest.first, dest.second,
-                       Arg1()*Param(scale));
+        vigra::transformImage(dest.first, dest.first + vigra::Diff2D(info.width(), info.height()), dest.second,
+            dest.first, dest.second,
+            vigra::functor::Arg1()*vigra::functor::Param(scale));
     }
 }
 
@@ -516,13 +514,13 @@ ImageCache::EntryPtr ImageCache::getImage(const std::string & filename)
         return it->second;
     } else {
         if (m_progress) {
-            m_progress->pushTask(AppBase::ProgressTask("Loading image: "+hugin_utils::stripPath(filename), "", 0));
+            m_progress->setMessage("Loading image:", hugin_utils::stripPath(filename));
         }
         
         EntryPtr e = loadImageSafely(filename);
         
         if (m_progress) {
-            m_progress->popTask();
+            m_progress->taskFinished();
         }
         
         if (!e.get())
@@ -545,10 +543,15 @@ ImageCache::EntryPtr ImageCache::loadImageSafely(const std::string & filename)
     ImageCacheRGB16Ptr img16(new vigra::UInt16RGBImage);
     ImageCacheRGBFloatPtr imgFloat(new vigra::FRGBImage);
     ImageCache8Ptr mask(new vigra::BImage);
+    ImageCacheICCProfile iccProfile(new vigra::ImageImportInfo::ICCProfile);
 
     try {
         vigra::ImageImportInfo info(filename.c_str());
 
+        if (!info.getICCProfile().empty())
+        {
+            *iccProfile = info.getICCProfile();
+        };
         int bands = info.numBands();
         int extraBands = info.numExtraBands();
         const char * pixelType = info.getPixelType();
@@ -605,6 +608,50 @@ ImageCache::EntryPtr ImageCache::loadImageSafely(const std::string & filename)
                 copyImage(srcImageRange(*imgFloat, vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)),
                           destImage(*imgFloat, vigra::VectorComponentAccessor<vigra::RGBValue<float> >(2)));
             }
+        } else if ( bands == 2 && extraBands==1) {
+            mask->resize(info.size());
+            // load and convert image to 8 bit, if needed
+            if (strcmp(pixelType, "UINT8") == 0 ) {
+                vigra::importImageAlpha(info, destImage(*img8,
+                                                   vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt8> >(0)),
+                                                   destImage(*mask));
+                copyImage(srcImageRange(*img8, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt8> >(0)),
+                          destImage(*img8, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt8> >(1)));
+                copyImage(srcImageRange(*img8, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt8> >(0)),
+                          destImage(*img8, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt8> >(2)));
+            } else if (strcmp(pixelType, "UINT16") == 0 ) {
+                vigra::importImageAlpha(info, destImage(*img16,
+                                                   vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt16> >(0)),
+                                                   destImage(*mask));
+                copyImage(srcImageRange(*img16, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt16> >(0)),
+                          destImage(*img16, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt16> >(1)));
+                copyImage(srcImageRange(*img16, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt16> >(0)),
+                          destImage(*img16, vigra::VectorComponentAccessor<vigra::RGBValue<vigra::UInt16> >(2)));
+            } else {
+                if (strcmp(pixelType, "INT16") == 0 ) {
+                    importAndConvertAlphaImage<vigra::Int16> (info, destImage(*imgFloat,
+                            vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)), destImage(*mask), pixelType);
+                } else if (strcmp(pixelType, "UINT32") == 0 ) {
+                    importAndConvertAlphaImage<vigra::UInt32>(info, destImage(*imgFloat,
+                            vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)), destImage(*mask), pixelType);
+                } else if (strcmp(pixelType, "INT32") == 0 ) {
+                    importAndConvertAlphaImage<vigra::Int32>(info, destImage(*imgFloat,
+                            vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)), destImage(*mask), pixelType);
+                } else if (strcmp(pixelType, "FLOAT") == 0 ) {
+                    importAndConvertAlphaImage<float>(info, destImage(*imgFloat,
+                            vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)), destImage(*mask), pixelType);
+                } else if (strcmp(pixelType, "DOUBLE") == 0 ) {
+                    importAndConvertAlphaImage<double>(info, destImage(*imgFloat,
+                            vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)), destImage(*mask), pixelType);
+                } else {
+                    DEBUG_ERROR("Unsupported pixel type: " << pixelType);
+                    return EntryPtr();
+                }
+                copyImage(srcImageRange(*imgFloat, vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)),
+                          destImage(*imgFloat, vigra::VectorComponentAccessor<vigra::RGBValue<float> >(1)));
+                copyImage(srcImageRange(*imgFloat, vigra::VectorComponentAccessor<vigra::RGBValue<float> >(0)),
+                          destImage(*imgFloat, vigra::VectorComponentAccessor<vigra::RGBValue<float> >(2)));
+            }
         } else if (bands == 3 && extraBands == 0) {
             DEBUG_DEBUG( pixelType);
             // load and convert image to 8 bit, if needed
@@ -659,7 +706,7 @@ ImageCache::EntryPtr ImageCache::loadImageSafely(const std::string & filename)
        return EntryPtr();
     }
 
-    return EntryPtr(new Entry(img8, img16, imgFloat, mask, pixelTypeStr));
+    return EntryPtr(new Entry(img8, img16, imgFloat, mask, iccProfile, pixelTypeStr));
 }
 
 ImageCache::EntryPtr ImageCache::getImageIfAvailable(const std::string & filename)
@@ -682,14 +729,14 @@ ImageCache::EntryPtr ImageCache::getSmallImage(const std::string & filename)
     softFlush();
     std::map<std::string, EntryPtr>::iterator it;
     // "_small" is only used internally
-    string name = filename + string(":small");
+    std::string name = filename + std::string(":small");
     it = images.find(name);
     if (it != images.end()) {
         return it->second;
     } else {
         if (m_progress)
         {
-            m_progress->pushTask(AppBase::ProgressTask("Scaling image: "+hugin_utils::stripPath(filename), "", 0));
+            m_progress->setMessage("Scaling image:", hugin_utils::stripPath(filename));
         }
         DEBUG_DEBUG("creating small image " << name );
         EntryPtr entry = getImage(filename);
@@ -699,7 +746,7 @@ ImageCache::EntryPtr ImageCache::getSmallImage(const std::string & filename)
         images[name] = small_entry;
         DEBUG_INFO ( "created small image: " << name);
         if (m_progress) {
-            m_progress->popTask();
+            m_progress->taskFinished();
         }
         return small_entry;
     }
@@ -733,6 +780,11 @@ ImageCache::EntryPtr ImageCache::loadSmallImageSafely(EntryPtr entry)
     }
     EntryPtr e(new Entry);
     e->origType = entry->origType;
+    // also copy icc profile
+    if (!entry->iccProfile->empty())
+    {
+        *(e->iccProfile) = *(entry->iccProfile);
+    };
     // TODO: fix bug with mask reduction
     vigra::BImage fullsizeMask = *(entry->mask);
     if (entry->imageFloat->width() != 0 ) {
@@ -768,7 +820,7 @@ ImageCache::EntryPtr ImageCache::getSmallImageIfAvailable(const std::string & fi
     softFlush();
     std::map<std::string, EntryPtr>::iterator it;
     // "_small" is only used internally
-    string name = filename + string(":small");
+    std::string name = filename + std::string(":small");
     it = images.find(name);
     if (it != images.end()) {
         return it->second;
@@ -841,7 +893,7 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
          it != m_smallRequests.end();)
     {
         std::map<std::string, RequestPtr>::iterator next_it = it;
-        next_it++;
+        ++next_it;
         if (it->second.unique()) {
             // Last copy of the request is in the list.
             // Anything requesting it must have given up waiting.
@@ -850,7 +902,11 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
         } else if (getSmallImageIfAvailable(it->first).get()) {
             // already loaded.
             // signal to anything waiting and remove from the list.
-            it->second->ready(getSmallImage(it->first), it->first, true);
+            while (!it->second->ready.empty())
+            {
+                it->second->ready.front()(getSmallImage(it->first), it->first, true);
+                it->second->ready.erase(it->second->ready.begin());
+            };
             m_smallRequests.erase(it);
         }
         it = next_it;
@@ -859,7 +915,7 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
          it != m_requests.end();)
     {
         std::map<std::string, RequestPtr>::iterator next_it = it;
-        next_it++;
+        ++next_it;
         if (it->second.unique()) {
             // The last copy of the request is in the list of requests.
             // Anything that requested it must have given up waiting.
@@ -868,7 +924,11 @@ void ImageCache::postEvent(RequestPtr request, EntryPtr entry)
         } else if (getImageIfAvailable(it->first).get()) {
             // already loaded.
             // Signal to anything waiting.
-            it->second->ready(getImage(it->first), it->first, false);
+            while (!it->second->ready.empty())
+            {
+                it->second->ready.front()(getImage(it->first), it->first, false);
+                it->second->ready.erase(it->second->ready.begin());
+            };
             m_requests.erase(it);
         }
         it = next_it;
@@ -891,7 +951,12 @@ void ImageCache::spawnAsyncThread()
         {
             DEBUG_DEBUG("Not staring a thread to load an image, since no images are wanted.");
         } else {
+#ifdef USE_CXX11_THREAD
+            std::thread thread(loadSafely, it->second, EntryPtr());
+            thread.detach();
+#else
             boost::thread(loadSafely, it->second, EntryPtr());
+#endif
         }
     } else {
         // got a small image request, check if its larger version has loaded.
@@ -901,10 +966,20 @@ void ImageCache::spawnAsyncThread()
         {
             // the larger one is needed to generate it.
             RequestPtr request(new Request(filename, false));
+#ifdef USE_CXX11_THREAD
+            std::thread thread(loadSafely, request, EntryPtr());
+            thread.detach();
+#else
             boost::thread(loadSafely, request, EntryPtr());
+#endif
         } else {
             // we have the large image.
+#ifdef USE_CXX11_THREAD
+            std::thread thread(loadSafely, it->second, large);
+            thread.detach();
+#else
             boost::thread(loadSafely, it->second, large);
+#endif
         }
     }
     // thread should be processing the image asynchronously now.
