@@ -18,23 +18,15 @@
  *  Lesser General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public
- *  License along with this software; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  License along with this software. If not, see
+ *  <http://www.gnu.org/licenses/>.
  *
  */
 
 #include <iostream>
 #include <iomanip>
 
-#if !defined Hugin_shared || !defined _WINDOWS
-#define GLEW_STATIC
-#endif
 #include <GL/glew.h>
-#ifdef __APPLE__
-  #include <GLUT/glut.h>
-#else
-  #include <GL/glut.h>
-#endif
 
 #include <string.h>
 #ifdef _WIN32
@@ -72,7 +64,8 @@ using std::vector;
 
 using vigra::Rect2D;
 
-#define CHECK_GL() checkGLErrors(__LINE__, __FILE__)
+#define ___NCFILE___ ((char *) __FILE__)
+#define CHECK_GL() checkGLErrors(__LINE__, ___NCFILE___)
 
 
 static GLenum XGLMap[] = {
@@ -85,7 +78,7 @@ static GLenum XGLMap[] = {
     GL_LUMINANCE, GL_RGB, GL_LUMINANCE_ALPHA, GL_RGBA
 };
 
-static char* XGLStringMap[] = {
+static const char* XGLStringMap[] = {
     // gltypes
     "GL_BYTE", "GL_UNSIGNED_BYTE", "GL_SHORT", "GL_UNSIGNED_SHORT", "GL_INT", "GL_UNSIGNED_INT", "GL_FLOAT",
     // Internalformats
@@ -115,26 +108,63 @@ static const char* AlphaCompositeKernelSource = {
 };
 
 static void checkGLErrors(int line, char* file) {
-    GLenum errCode;
-    if ((errCode = glGetError()) != GL_NO_ERROR) {
-        cerr << "nona: GL error in " << file << ":" << line << ": " << gluErrorString(errCode) << endl;
+    GLenum errCode = glGetError();
+    if (errCode != GL_NO_ERROR)
+    {
+        while (errCode != GL_NO_ERROR)
+        {
+            const GLubyte* message = gluErrorString(errCode);
+            std::cerr << "nona: GL error in " << file << ":" << line << std::endl;
+            if (message)
+            {
+#ifdef _WIN32
+                const char* messageChar = reinterpret_cast<const char*>(message);
+                size_t size = strlen(messageChar);
+                LPSTR OEMmessageBuffer = (LPSTR)LocalAlloc(LPTR, (size + 1)*sizeof(char));
+                if (OEMmessageBuffer)
+                {
+                    if (CharToOemBuff(messageChar, OEMmessageBuffer, size))
+                    {
+                        std::cerr << OEMmessageBuffer << " (0x" << std::hex << errCode << ")" << std::endl;
+                    }
+                    else
+                    {
+                        std::cerr << message << " (0x" << std::hex << errCode << ")" << std::endl;
+                    };
+                }
+                else
+                {
+                    std::cerr << message << " (0x" << std::hex << errCode << ")" << std::endl;
+                };
+                LocalFree(OEMmessageBuffer);
+#else
+                std::cerr << message << " (0x" << std::hex << errCode << ")" << std::endl;
+#endif
+            }
+            else
+            {
+                std::cerr << "Error code: 0x" << std::hex << errCode << std::endl;
+            }
+            errCode = glGetError();
+        };
         exit(1);
     }
 }
 
 static void printInfoLog(GLhandleARB obj) {
-  GLint infologLength = 0;
-  GLint charsWritten = 0;
-  char *infoLog;
+    GLint infologLength = 0;
+    GLint charsWritten = 0;
     glGetObjectParameterivARB(obj, GL_OBJECT_INFO_LOG_LENGTH_ARB, &infologLength);
     if (infologLength > 1) {
-        infoLog = new char[infologLength];
+        char* infoLog = new char[infologLength];
         glGetInfoLogARB(obj, infologLength, &charsWritten, infoLog);
         cout << "nona: GL info log:" << endl << infoLog << endl << endl;
         delete[] infoLog;
     }
 }
 
+static bool printDebug=false;
+
 static bool checkFramebufferStatus(int line, char* file) {
     GLenum status;
     status = (GLenum) glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
@@ -187,7 +217,8 @@ static void compileGLSL(const char* programName,
         exit(1);
     }
 
-    printInfoLog(shaderObject);
+    if(printDebug)
+        printInfoLog(shaderObject);
 
     glAttachObjectARB(programObject, shaderObject);
     glLinkProgramARB(programObject);
@@ -199,7 +230,8 @@ static void compileGLSL(const char* programName,
         exit(1);
     }
 
-    printInfoLog(programObject);
+    if(printDebug)
+        printInfoLog(programObject);
 }
 
 static void makeChunks(const int width,
@@ -264,6 +296,11 @@ static void makeChunks(const int width,
 namespace vigra_ext
 {
 
+void SetGPUDebugMessages(const bool doPrint)
+{
+    printDebug=doPrint;
+};
+
 bool transformImageGPUIntern(const std::string& coordXformGLSL,
                              const std::string& interpolatorGLSL,
                              const int interpolatorSize,
@@ -283,56 +320,80 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                              const int destAlphaGLType,
                              const bool warparound)
 {
-    long t1, t2, t3, t4, t5, t6, t7, t8, t9, t10, t11, t12, t13, t14, t15, t16, t17, t18, t19, t20, t21;
-    t1=getms();
+    long t1, t3, t5, t6, t7, t9, t10, t11, t12, t13, t14, t15, t16, t17, t18, t19, t21;
 
     const int xstart = destUL.x;
     const int xend   = destUL.x + destSize.x;
     const int ystart = destUL.y;
     const int yend   = destUL.y + destSize.y;
 
-    cout << "destStart=[" << xstart << ", " << ystart << "]" << endl
-         << "destEnd=[" << xend << ", " << yend << "]" << endl
-         << "destSize=[" << destSize << "]" << endl
-         << "srcSize=[" << srcSize << "]" << endl;
+    if(printDebug)
+    {
+        t1=getms();
+        cout << "destStart=[" << xstart << ", " << ystart << "]" << endl
+             << "destEnd=[" << xend << ", " << yend << "]" << endl
+             << "destSize=[" << destSize << "]" << endl
+             << "srcSize=[" << srcSize << "]" << endl
+             << "srcBuffer=" << srcBuffer << endl
+             << "srcAlphaBuffer=" << srcAlphaBuffer << endl
+             << "destBuffer=" << destBuffer << endl
+             << "destAlphaBuffer=" << destAlphaBuffer << endl
+             << "destGLInternalFormat=" << XGLStringMap[destGLInternalFormat] << endl
+             << "destGLFormat=" << XGLStringMap[destGLFormat] << endl
+             << "destGLType=" << XGLStringMap[destGLType] << endl
+             << "srcGLInternalFormat=" << XGLStringMap[srcGLInternalFormat] << endl
+             << "srcGLFormat=" << XGLStringMap[srcGLFormat] << endl
+             << "srcGLType=" << XGLStringMap[srcGLType] << endl
+             << "srcAlphaGLType=" << XGLStringMap[srcAlphaGLType] << endl
+             << "destAlphaGLType=" << XGLStringMap[destAlphaGLType] << endl
+             << endl
+             << "warparound=" << warparound << endl;
+    };
 
     vigra_precondition((srcSize.x % 8) == 0, "src image width not a multiple of 8");
     vigra_precondition((destSize.x % 8) == 0, "dest image width not a multiple of 8");
 
-    cout << "srcBuffer=" << srcBuffer << endl
-         << "srcAlphaBuffer=" << srcAlphaBuffer << endl
-         << "destBuffer=" << destBuffer << endl
-         << "destAlphaBuffer=" << destAlphaBuffer << endl;
-
     vigra_precondition((reinterpret_cast<const uintptr_t>(srcBuffer) & 0x7) == 0, "src image buffer not 8-byte aligned");
     vigra_precondition((reinterpret_cast<const uintptr_t>(srcAlphaBuffer) & 0x7) == 0, "src alpha image buffer not 8-byte aligned");
     vigra_precondition((reinterpret_cast<const uintptr_t>(destBuffer) & 0x7) == 0, "dest image buffer not 8-byte aligned");
     vigra_precondition((reinterpret_cast<const uintptr_t>(destAlphaBuffer) & 0x7) == 0, "dest alpha image buffer not 8-byte aligned");
 
-    cout << "destGLInternalFormat=" << XGLStringMap[destGLInternalFormat] << endl
-         << "destGLFormat=" << XGLStringMap[destGLFormat] << endl
-         << "destGLType=" << XGLStringMap[destGLType] << endl
-         << "srcGLInternalFormat=" << XGLStringMap[srcGLInternalFormat] << endl
-         << "srcGLFormat=" << XGLStringMap[srcGLFormat] << endl
-         << "srcGLType=" << XGLStringMap[srcGLType] << endl
-         << "srcAlphaGLType=" << XGLStringMap[srcAlphaGLType] << endl
-         << "destAlphaGLType=" << XGLStringMap[destAlphaGLType] << endl;
-
-    cout << "warparound=" << warparound << endl;
-
     const char* const gpuVendor = reinterpret_cast<const char*>(glGetString(GL_VENDOR));
     const bool needsAtanWorkaround = (strncmp(gpuVendor, "ATI", 3) == 0);
 
-    cout << "needsAtanWorkaround=" << needsAtanWorkaround << endl;
+    if(printDebug)
+        cout << "needsAtanWorkaround=" << needsAtanWorkaround << endl;
 
     GLint maxTextureSize;
     glGetIntegerv(GL_MAX_TEXTURE_SIZE, &maxTextureSize);
-    cout << "maxTextureSize=" << maxTextureSize << endl;
+    if(printDebug)
+        cout << "maxTextureSize=" << maxTextureSize << endl;
 
     // Artificial limit: binding big textures to fbos seems to be very slow.
     //maxTextureSize = 2048;
 
-    const long long int GpuMemoryInBytes = 512 << 20;
+    long long int GpuMemoryInBytes = 512 << 20;
+    // read memory size, not implemented on all graphic cards
+    {
+        // for nvidia cards
+#define GL_GPU_MEM_INFO_TOTAL_AVAILABLE_MEM_NVX 0x9048
+        GLint total_mem_kb = 0;
+        glGetIntegerv(GL_GPU_MEM_INFO_TOTAL_AVAILABLE_MEM_NVX, &total_mem_kb);
+        if (glGetError() == GL_NO_ERROR)
+        {
+            GpuMemoryInBytes = total_mem_kb * 1024ll;
+        };
+    };
+    {
+        //for amd/ati cards
+#define TEXTURE_FREE_MEMORY_ATI 0x87FC
+        GLint param[4];
+        glGetIntegerv(TEXTURE_FREE_MEMORY_ATI, param);
+        if (glGetError() == GL_NO_ERROR)
+        {
+            GpuMemoryInBytes = param[0] * 1024ll;
+        };
+    }
     const double SourceAllocationRatio = 0.7;
 
     const int bytesPerSourcePixel = BytesPerPixel[srcGLInternalFormat]
@@ -355,30 +416,36 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     const long long int maxDestPixels = gpuMemoryRemaining / bytesPerDestPixel;
 
     vector<Rect2D> destChunks;
-    makeChunks(destSize.x, destSize.y, 2048, maxDestPixels, destChunks);
+    makeChunks(destSize.x, destSize.y, maxTextureSize, maxDestPixels, destChunks);
 
     const long long int totalGpuMemoryUsed = (sourceChunks[0].area() * bytesPerSourcePixel) + (destChunks[0].area() * bytesPerDestPixel);
     vigra_assert(totalGpuMemoryUsed <= GpuMemoryInBytes,
                  "failed to subdivide source and dest images into pieces small enough to fit in gpu memory.");
 
-    cout << "Source chunks:" << endl;
-    for (vector<Rect2D>::iterator rI = sourceChunks.begin(); rI != sourceChunks.end(); ++rI) {
-        cout << "    " << *rI << endl;
-    }
-    cout << "Dest chunks:" << endl;
-    for (vector<Rect2D>::iterator rI = destChunks.begin(); rI != destChunks.end(); ++rI) {
-        cout << "    " << *rI << endl;
-    }
-    cout << "Total GPU memory used: " << totalGpuMemoryUsed << endl;
+    if(printDebug)
+    {
+        cout << "Source chunks:" << endl;
+        for (vector<Rect2D>::iterator rI = sourceChunks.begin(); rI != sourceChunks.end(); ++rI) {
+            cout << "    " << *rI << endl;
+        }
+        cout << "Dest chunks:" << endl;
+        for (vector<Rect2D>::iterator rI = destChunks.begin(); rI != destChunks.end(); ++rI) {
+            cout << "    " << *rI << endl;
+        }
+        cout << "Total GPU memory used: " << totalGpuMemoryUsed << endl;
+    };
 
 
     const int TextureFetchesPerInterpolatorPass = 16;
     vector<Rect2D> interpolatorChunks;
     makeChunks(interpolatorSize, interpolatorSize, TextureFetchesPerInterpolatorPass, TextureFetchesPerInterpolatorPass, interpolatorChunks);
-    cout << "Interpolator chunks:" << endl;
-    for (vector<Rect2D>::iterator rI = interpolatorChunks.begin(); rI != interpolatorChunks.end(); ++rI) {
-        cout << "    " << *rI << endl;
-    }
+    if(printDebug)
+    {
+        cout << "Interpolator chunks:" << endl;
+        for (vector<Rect2D>::iterator rI = interpolatorChunks.begin(); rI != interpolatorChunks.end(); ++rI) {
+            cout << "    " << *rI << endl;
+        }
+    };
     bool allInterpolatorChunksAreEqual = true;
     const Rect2D& firstInterpolatorChunk = interpolatorChunks.front();
     for (vector<Rect2D>::iterator rI = ++(interpolatorChunks.begin()); rI != interpolatorChunks.end(); ++rI) {
@@ -444,7 +511,8 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
 
     std::string coordXformKernelSourceString = oss.str();
     const char* coordXformKernelSource = coordXformKernelSourceString.c_str();
-    cout << coordXformKernelSource;
+    if(printDebug)
+        cout << coordXformKernelSource;
 
     GLhandleARB coordXformProgramObject;
     GLhandleARB coordXformShaderObject;
@@ -562,7 +630,8 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
 
     std::string interpolatorKernelSourceString = oss.str();
     const char* interpolatorKernelSource = interpolatorKernelSourceString.c_str();
-    cout << interpolatorKernelSource;
+    if(printDebug)
+        cout << interpolatorKernelSource;
 
     GLhandleARB interpolatorProgramObject;
     GLhandleARB interpolatorShaderObject;
@@ -612,7 +681,8 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
 
     std::string normalizationPhotometricKernelSourceString = oss.str();
     const char* normalizationPhotometricKernelSource = normalizationPhotometricKernelSourceString.c_str();
-    cout << normalizationPhotometricKernelSource;
+    if(printDebug)
+        cout << normalizationPhotometricKernelSource;
 
     GLhandleARB normalizationPhotometricProgramObject;
     GLhandleARB normalizationPhotometricShaderObject;
@@ -628,7 +698,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     GLuint invLutTexture;
     GLint invLutTextureParam;
     if (!invLut.empty()) {
-        invLutTextureData = new float[invLut.size() * 4];
+        invLutTextureData = new float[invLut.size() * 2];
         for (int i = 0; i < invLut.size(); ++i) {
             invLutTextureData[2*i] = invLut[i];
             invLutTextureData[2*i+1] = ((i + 1) < invLut.size()) ? invLut[i+1] : invLut[i];
@@ -640,7 +710,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP);
-        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_LUMINANCE_ALPHA32F_ARB, invLut.size(), 2, 0, GL_LUMINANCE_ALPHA, GL_FLOAT, invLutTextureData);
+        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_LUMINANCE_ALPHA32F_ARB, invLut.size(), 1, 0, GL_LUMINANCE_ALPHA, GL_FLOAT, invLutTextureData);
         CHECK_GL();
     }
 
@@ -648,7 +718,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     GLuint destLutTexture;
     GLint destLutTextureParam;
     if (!destLut.empty()) {
-        destLutTextureData = new float[destLut.size() * 4];
+        destLutTextureData = new float[destLut.size() * 2];
         for (int i = 0; i < destLut.size(); ++i) {
             destLutTextureData[2*i] = destLut[i];
             destLutTextureData[2*i+1] = ((i + 1) < destLut.size()) ? destLut[i+1] : destLut[i];
@@ -660,13 +730,16 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP);
-        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_LUMINANCE_ALPHA32F_ARB, destLut.size(), 2, 0, GL_LUMINANCE_ALPHA, GL_FLOAT, destLutTextureData);
+        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_LUMINANCE_ALPHA32F_ARB, destLut.size(), 1, 0, GL_LUMINANCE_ALPHA, GL_FLOAT, destLutTextureData);
         CHECK_GL();
     }
 
     glFinish();
-    t21=getms();
-    cout << "gpu shader program compile time = " << ((t21 - t1)/1000.0) << endl;
+    if(printDebug)
+    {
+        t21=getms();
+        cout << "gpu shader program compile time = " << ((t21 - t1)/1000.0) << endl;
+    };
 
     // General GL setup
     glPixelStorei(GL_PACK_ALIGNMENT, 8);
@@ -702,7 +775,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
     glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP);
     glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP);
-    glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_LUMINANCE_ALPHA32F_ARB, destChunks[0].width(), destChunks[0].height() + destOdd, 0, GL_LUMINANCE_ALPHA, GL_FLOAT, NULL);
+    glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA32F_ARB, destChunks[0].width(), destChunks[0].height() + destOdd, 0, GL_LUMINANCE_ALPHA, GL_FLOAT, NULL);
     CHECK_GL();
 
     // Setup coordinate framebuffer
@@ -710,7 +783,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     CHECK_GL();
     glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, coordTexture, 0);
     CHECK_GL();
-    if (!checkFramebufferStatus(__LINE__, __FILE__)) {
+    if (!checkFramebufferStatus(__LINE__, ___NCFILE___)) {
         exit(1);
     }
 
@@ -738,7 +811,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, accumFB);
     glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, accumTextures[0], 0);
     glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT1_EXT, GL_TEXTURE_RECTANGLE_ARB, accumTextures[1], 0);
-    if (!checkFramebufferStatus(__LINE__, __FILE__)) {
+    if (!checkFramebufferStatus(__LINE__, ___NCFILE___)) {
         exit(1);
     }
 
@@ -757,7 +830,7 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
 
     glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, srcFB);
     glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, srcTexture, 0);
-    if (!checkFramebufferStatus(__LINE__, __FILE__)) {
+    if (!checkFramebufferStatus(__LINE__, ___NCFILE___)) {
         exit(1);
     }
 
@@ -801,11 +874,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP);
-        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, XGLMap[destGLTransferFormat], destChunks[0].width(), destChunks[0].height() + destOdd, 0, XGLMap[destGLFormat], XGLMap[destGLType], NULL);
+        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, XGLMap[destGLInternalFormat], destChunks[0].width(), destChunks[0].height() + destOdd, 0, XGLMap[destGLFormat], XGLMap[destGLType], NULL);
 
         glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, destFB);
         glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, destTexture, 0);
-        if (!checkFramebufferStatus(__LINE__, __FILE__)) {
+        if (!checkFramebufferStatus(__LINE__, ___NCFILE___)) {
             exit(1);
         }
     }
@@ -820,27 +893,30 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_S, GL_CLAMP);
         glTexParameteri(GL_TEXTURE_RECTANGLE_ARB, GL_TEXTURE_WRAP_T, GL_CLAMP);
-        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_ALPHA, destChunks[0].width(), destChunks[0].height() + destOdd, 0, GL_ALPHA, XGLMap[destAlphaGLType], NULL);
+        glTexImage2D(GL_TEXTURE_RECTANGLE_ARB, 0, GL_RGBA, destChunks[0].width(), destChunks[0].height() + destOdd, 0, GL_ALPHA, XGLMap[destAlphaGLType], NULL);
         CHECK_GL();
 
         glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, destAlphaFB);
         glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, GL_COLOR_ATTACHMENT0_EXT, GL_TEXTURE_RECTANGLE_ARB, destAlphaTexture, 0);
-        if (!checkFramebufferStatus(__LINE__, __FILE__)) {
+        if (!checkFramebufferStatus(__LINE__, ___NCFILE___)) {
             exit(1);
         }
     }
 
 
     glFinish();
-    t2=getms();
-    cout << "gpu shader texture/framebuffer setup time = " << ((t2-t21)/1000.0) << endl;
+    if(printDebug)
+    {
+        cout << "gpu shader texture/framebuffer setup time = " << ((getms()-t21)/1000.0) << endl;
+    };
 
     // Render each dest chunk
     int destChunkNumber = 0;
     for (vector<Rect2D>::iterator dI = destChunks.begin(); dI != destChunks.end(); ++dI, ++destChunkNumber) {
 
         glFinish();
-        t3=getms();
+        if(printDebug)
+            t3=getms();
 
         // Render coord image
         glUseProgramObjectARB(coordXformProgramObject);
@@ -857,8 +933,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         CHECK_GL();
 
         glFinish();
-        t4=getms();
-        cout << "gpu dest chunk=" << *dI << " coord image render time = " << ((t4-t3)/1000.0) << endl;
+        if(printDebug)
+        {
+            long t4=getms();
+            cout << "gpu dest chunk=" << *dI << " coord image render time = " << ((t4-t3)/1000.0) << endl;
+        };
 
         // Multipass rendering of dest image
         int pass = 0;
@@ -866,7 +945,8 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
 
             if (destChunkNumber == 0 || sourceChunks.size() > 1) {
                 glFinish();
-                t5=getms();
+                if(printDebug)
+                    t5=getms();
 
                 glPixelStorei(GL_UNPACK_ROW_LENGTH, srcSize.x);
                 glPixelStorei(GL_UNPACK_SKIP_PIXELS, sI->left());
@@ -885,8 +965,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                     CHECK_GL();
 
                     glFinish();
-                    t6=getms();
-                    cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src upload = " << ((t6-t5)/1000.0) << endl;
+                    if(printDebug)
+                    {
+                        t6=getms();
+                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src upload = " << ((t6-t5)/1000.0) << endl;
+                    };
 
                     if (srcAlphaBuffer != NULL) {
                         // Upload to srcAlphaTexture and composite to srcTexture.
@@ -900,8 +983,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                         CHECK_GL();
 
                         glFinish();
-                        t7=getms();
-                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src alpha upload = " << ((t7-t6)/1000.0) << endl;
+                        if(printDebug)
+                        {
+                            t7=getms();
+                            cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src alpha upload = " << ((t7-t6)/1000.0) << endl;
+                        };
 
                         glPolygonMode(GL_FRONT, GL_FILL);
                         glBegin(GL_QUADS);
@@ -913,8 +999,10 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                         CHECK_GL();
 
                         glFinish();
-                        t8=getms();
-                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src+alpha render = " << ((t8-t7)/1000.0) << endl;
+                        if(printDebug)
+                        {
+                            cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src+alpha render = " << ((getms()-t7)/1000.0) << endl;
+                        };
                     }
                 }
                 else {
@@ -930,8 +1018,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                     CHECK_GL();
 
                     glFinish();
-                    t6=getms();
-                    cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src upload = " << ((t6-t5)/1000.0) << endl;
+                    if(printDebug)
+                    {
+                        t6=getms();
+                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src upload = " << ((t6-t5)/1000.0) << endl;
+                    };
 
                     if (srcAlphaBuffer != NULL) {
                         // Upload to srcAlphaTexture
@@ -943,8 +1034,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                         CHECK_GL();
 
                         glFinish();
-                        t7=getms();
-                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src alpha upload = " << ((t7- t6)/1000.0) << endl;
+                        if(printDebug)
+                        {
+                            t7=getms();
+                            cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src alpha upload = " << ((t7- t6)/1000.0) << endl;
+                        };
 
                         glPolygonMode(GL_FRONT, GL_FILL);
                         glBegin(GL_QUADS);
@@ -962,8 +1056,10 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                         CHECK_GL();
 
                         glFinish();
-                        t8=getms();
-                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src+alpha render = " << ((t8-t7)/1000.0) << endl;
+                        if(printDebug)
+                        {
+                            cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src+alpha render = " << ((getms()-t7)/1000.0) << endl;
+                        };
                     }
                     else {
                         glPolygonMode(GL_FRONT, GL_FILL);
@@ -984,14 +1080,18 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                         CHECK_GL();
 
                         glFinish();
-                        t7=getms();
-                        cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src render = " << ((t7-t6)/1000.0) << endl;
+                        if(printDebug)
+                        {
+                            t7=getms();
+                            cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " src render = " << ((t7-t6)/1000.0) << endl;
+                        };
                     }
                 }
             }
 
             glFinish();
-            t9=getms();
+            if(printDebug)
+                t9=getms();
 
             // Render dest image
             glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, accumFB);
@@ -1038,8 +1138,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                 glDrawBuffer((pass & 1) ? GL_COLOR_ATTACHMENT1_EXT : GL_COLOR_ATTACHMENT0_EXT);
 
                 glFinish();
-                t10=getms();
-                cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " interpolation chunk=" << *iI << " setup = " << ((t10-t9)/1000.0) << endl;
+                if(printDebug)
+                {
+                    t10=getms();
+                    cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " interpolation chunk=" << *iI << " setup = " << ((t10-t9)/1000.0) << endl;
+                };
 
                 glPolygonMode(GL_FRONT, GL_FILL);
                 glBegin(GL_QUADS);
@@ -1051,9 +1154,12 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
                 CHECK_GL();
 
                 glFinish();
-                t11=getms();
-                t9=getms();
-                cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " interpolation chunk=" << *iI << " render = " << ((t11-t10)/1000.0) << endl;
+                if(printDebug)
+                {
+                    t11=getms();
+                    t9=getms();
+                    cout << "gpu dest chunk=" << *dI << " source chunk=" << *sI << " interpolation chunk=" << *iI << " render = " << ((t11-t10)/1000.0) << endl;
+                };
 
             } // next interpolation chunk
 
@@ -1093,8 +1199,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         glDrawBuffer((pass & 1) ? GL_COLOR_ATTACHMENT1_EXT : GL_COLOR_ATTACHMENT0_EXT);
         
         glFinish();
-        t12=getms();
-        cout << "gpu dest chunk=" << *dI << " normalization setup = " << ((t12-t11)/1000.0) << endl;
+        if(printDebug)
+        {
+            t12=getms();
+            cout << "gpu dest chunk=" << *dI << " normalization setup = " << ((t12-t11)/1000.0) << endl;
+        };
 
         glPolygonMode(GL_FRONT, GL_FILL);
         glBegin(GL_QUADS);
@@ -1106,8 +1215,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
         CHECK_GL();
 
         glFinish();
-        t13=getms();
-        cout << "gpu dest chunk=" << *dI << " normalization render = " << ((t13-t12)/1000.0) << endl;
+        if(printDebug)
+        {
+            t13=getms();
+            cout << "gpu dest chunk=" << *dI << " normalization render = " << ((t13-t12)/1000.0) << endl;
+        };
 
         pass++;
 
@@ -1125,11 +1237,15 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t14=getms();
-            cout << "gpu dest chunk=" << *dI << " rgb readback = " << ((t14-t13)/1000.0) << endl;
+            if(printDebug)
+            {
+                t14=getms();
+                cout << "gpu dest chunk=" << *dI << " rgb readback = " << ((t14-t13)/1000.0) << endl;
+            };
         }
         else {
             // Move output accumTexture to dest texture then readback.
+            glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
             glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, destFB);
             glUseProgramObjectARB(0);
 
@@ -1140,8 +1256,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t14=getms();
-            cout << "gpu dest chunk=" << *dI << " dest rgb disassembly setup = " << ((t14-t13)/1000.0) << endl;
+            if(printDebug)
+            {
+                t14=getms();
+                cout << "gpu dest chunk=" << *dI << " dest rgb disassembly setup = " << ((t14-t13)/1000.0) << endl;
+            };
 
             glPolygonMode(GL_FRONT, GL_FILL);
             glBegin(GL_QUADS);
@@ -1157,8 +1276,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t15=getms();
-            cout << "gpu dest chunk=" << *dI << " dest rgb disassembly render = " << ((t15-t14)/1000.0) << endl;
+            if(printDebug)
+            {
+                t15=getms();
+                cout << "gpu dest chunk=" << *dI << " dest rgb disassembly render = " << ((t15-t14)/1000.0) << endl;
+            };
 
             glReadBuffer(GL_COLOR_ATTACHMENT0_EXT);
             CHECK_GL();
@@ -1167,8 +1289,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t16=getms();
-            cout << "gpu dest chunk=" << *dI << " rgb readback = " << ((t16-t15)/1000.0) << endl;
+            if(printDebug)
+            {
+                t16=getms();
+                cout << "gpu dest chunk=" << *dI << " rgb readback = " << ((t16-t15)/1000.0) << endl;
+            };
         }
 
         if (destAlphaBuffer != NULL) {
@@ -1183,8 +1308,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t17=getms();
-            cout << "gpu dest chunk=" << *dI << " dest alpha disassembly setup = " << ((t17-t16)/1000.0) << endl;
+            if(printDebug)
+            {
+                t17=getms();
+                cout << "gpu dest chunk=" << *dI << " dest alpha disassembly setup = " << ((t17-t16)/1000.0) << endl;
+            };
 
             glPolygonMode(GL_FRONT, GL_FILL);
             glBegin(GL_QUADS);
@@ -1200,8 +1328,11 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t18=getms();
-            cout << "gpu dest chunk=" << *dI << " dest alpha disassembly render = " << ((t18-t17)/1000.0) << endl;
+            if(printDebug)
+            {
+                t18=getms();
+                cout << "gpu dest chunk=" << *dI << " dest alpha disassembly render = " << ((t18-t17)/1000.0) << endl;
+            };
 
             // Readback dest alpha chunk
             glReadBuffer(GL_COLOR_ATTACHMENT0_EXT);
@@ -1211,14 +1342,18 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
             CHECK_GL();
 
             glFinish();
-            t19=getms();
-            cout << "gpu dest chunk=" << *dI << " alpha readback = " << ((t19-t18)/1000.0) << endl;
+            if(printDebug)
+            {
+                t19=getms();
+                cout << "gpu dest chunk=" << *dI << " alpha readback = " << ((t19-t18)/1000.0) << endl;
+            };
         }
 
     } // next dest chunk
 
     glFinish();
-    t19=getms();
+    if(printDebug)
+        t19=getms();
 
     glDeleteTextures(2, accumTextures);
     glDeleteTextures(1, &coordTexture);
@@ -1249,9 +1384,12 @@ bool transformImageGPUIntern(const std::string& coordXformGLSL,
     glDeleteObjectARB(normalizationPhotometricProgramObject);
 
     glFinish();
-    t20=getms();
-    cout << "gpu destruct time = " << ((t20-t19)/1000.0) << endl;
-    cout << "gpu total time = " << ((t20-t1)/1000.0) << endl;
+    if(printDebug)
+    {
+        long t20=getms();
+        cout << "gpu destruct time = " << ((t20-t19)/1000.0) << endl;
+        cout << "gpu total time = " << ((t20-t1)/1000.0) << endl;
+    };
 
     return true;
 }
